! $Id: convolution.f90,v 1.3 2001/06/29 09:43:53 salam Exp $


!======================================================================
! Module exlcusively for communication between convolution and 
! any routine which might supply convolutions
!======================================================================
module convolution_communicator
  use types
  implicit none
  private

  integer, public , parameter :: cc_REAL=1,cc_VIRT=2,&
       & cc_REALVIRT=3,cc_DELTA =4 
  integer, public :: cc_piece
end module convolution_communicator








!======================================================================
! All the base types and routines for happy convolution
!
! Decide on the following policy: grid details will be copied rather
! than pointed to (avoids danger of object that is pointed to being 
! changed). Not clear if this is really the best policy, but we will
! give it a try...
!
! GPS 04/04/2000
!======================================================================
module convolution
  use types; use consts_dp; use assertions
  implicit none
  private

  !-------------------------------
  ! definition of a grid
  type grid_def
     real(dp) :: dy, ymax
     integer  :: ny, order
  end type grid_def

  !--------------------------------------------------
  ! used for abbreviated access to conv_EvalGridQuant
  
  
  type grid_conv
     type(grid_def)          :: gd
     real(dp),       pointer :: conv(:,:)
  end type grid_conv
  integer, parameter :: FULL=1, UPPR=2
  integer, parameter :: LIN_ORDER=0
  

  public :: grid_def, grid_conv


  !-- public routines --
  public :: conv_InitGridDef, conv_ValidateGD
  interface operator(==)
     module procedure conv_CmpGridDef
  end interface
  public :: operator(==)

  !-- quant routines -----------------------------------------------
  interface conv_AllocGridQuant
     module procedure conv_AllocGridQuant_1d,conv_AllocGridQuant_2d
  end interface
  public :: conv_AllocGridQuant
  interface conv_InitGridQuant
     module procedure conv_InitGridQuant_func, conv_InitGridQuant_func2d
  end interface
  public :: conv_InitGridQuant, conv_PrintGridQuant
  interface conv_PrintGridQuant
     module procedure conv_PrintGridQuant_1, conv_PrintGridQuant_2,&
          & conv_PrintGridQuant_3, conv_PrintGridQuant_4
  end interface
  public :: conv_MomGridQuant, conv_EvalGridQuant
  interface conv_DelGridQuant
     module procedure conv_DelGridQuant_1d, conv_DelGridQuant_2d
  end interface
  
  public :: conv_DelGridQuant

  !-- convolution routines -------------------------------------------
  interface conv_InitGridConv
     module procedure conv_InitGridConv_zero, conv_InitGridConv_func,&
          & conv_InitGridConv_gc
  end interface
  !-- keep following for historical reasons?
  interface conv_GridConvAdd
     module procedure conv_GridConvAdd_func, conv_GridConvAdd_gc
  end interface
  interface conv_AddGridConv
     module procedure conv_GridConvAdd_func, conv_GridConvAdd_gc
  end interface

  public :: conv_Seteps
  public :: conv_InitGridConv, conv_GridConvAdd, conv_DelGridConv
  public :: conv_AddGridConv, conv_MultGridConv

  !-- REMEMBER: .conv. seems to have a precedence similar to .or., .and.
  !             and so is lower precednece than any arithmetic operation
  interface operator(.conv.)
     module procedure conv_ConvQuant_mat, conv_ConvQuant_scalar
  end interface
  public :: operator(.conv.)
  !-- put in this version as well because to ease the writing of 
  !   expressions: * should have the right precedence properties
  !   where .conv. seems to have a very low precedence
  interface operator(*)
     module procedure conv_ConvQuant_mat, conv_ConvQuant_scalar
  end interface
  public :: operator(*)
  public :: conv_ConvConv

  !--------------------------
  type gdval
     type(grid_def) :: gd
     real(dp)       :: val
  end type gdval
  public :: gdval
  interface operator(.with.)
     module procedure conv_gdval_gdv, conv_gdval_vgd
  end interface
  public :: operator(.with.)  
  interface operator(.atx.)
     module procedure conv_EvalGridQuant_atx
  end interface
  public :: operator(.atx.)

  real(dp) :: eps=1e-7_dp
  !real(dp), parameter :: eps=1e-8_dp
  
contains
  
  !======================================================================
  ! Things just for Grid defs.
  subroutine conv_InitGridDef(gd,dy,ymax,order)
    type(grid_def), intent(out) :: gd
    real(dp),       intent(in)  :: dy, ymax
    integer,        intent(in), optional :: order

    gd%ymax = ymax
    gd%ny   = nint(ymax / dy)
    if (gd%ny < 2) then
       write(0,*) 'conv_NewGridDef: requested too small a number of bins'
       write(0,*) '                 dy and ymax were',dy,ymax
       stop
    end if
    
    gd%dy   = ymax / gd%ny
    if (abs(gd%dy/dy - one) > 0.001_dp) then
       write(0,*) 'conv_NewGridDef: requested dy of', dy
       write(0,*) '                 provided  dy of', gd%dy
    end if

    if (present(order)) then
       if (order < LIN_ORDER) then
          write(0,*) 'ERROR in convInitGridDef: order < 0&
               & is not valid in conv_InitGridDef'
          stop
       else if (order+1 > gd%ny) then
          write(0,*) &
               &'ERROR in convInitGridDef: order > number of grid points+1' 
       end if
       gd%order = order
    else
       gd%order = LIN_ORDER
    end if
    
  end subroutine conv_InitGridDef
  
  !----------------------------------------------------------------
  function conv_CmpGridDef(gd1,gd2) result(equal)
    type(grid_def), intent(in) :: gd1, gd2
    logical :: equal

    equal = (gd1%dy   == gd2%dy) .and. (gd1%ny == gd2%ny) .and.&
         &  (gd1%ymax == gd2%ymax)
  end function conv_CmpGridDef
  
  !-- useful to be able to check easily -------------------------
  subroutine conv_ValidateGD(gd1,gd2,source)
    type(grid_def), intent(in) :: gd1, gd2
    character(len=*), intent(in) :: source
    if (.not. (gd1 == gd2)) then
       write(0,*) 'Problem validating two grid defs in ',source
       stop
    end if
  end subroutine conv_ValidateGD


  !======================================================================
  ! Things just for grid quants
  subroutine conv_AllocGridQuant_1d(gd,gq)
    type(grid_def), intent(in) :: gd
    real(dp),       pointer    :: gq(:)
    integer :: istat
    ! this form of deallocate ought to be OK (?), but on lahey,
    ! compaq+condor (and perhaps elsewhere) it causes problems
    !deallocate(gq,stat=istat)
    allocate(gq(0:gd%ny))
  end subroutine conv_AllocGridQuant_1d

  !----------------------------------------------------------------------
  subroutine conv_AllocGridQuant_2d(gd,gq,nl,nh)
    type(grid_def), intent(in) :: gd
    real(dp),       pointer    :: gq(:,:)
    integer,        intent(in) :: nl,nh
    integer :: istat
    ! this form of deallocate ought to be OK (?), but on lahey,
    ! compaq+condor (and perhaps elsewhere) it causes problems
    !deallocate(gq,stat=istat)
    allocate(gq(0:gd%ny,nl:nh))
  end subroutine conv_AllocGridQuant_2d
  
  !----------------------------------------------------------
  subroutine conv_DelGridQuant_1d(gq)
    real(dp),       pointer    :: gq(:)
    integer :: istat
    deallocate(gq,stat=istat)
  end subroutine conv_DelGridQuant_1d
  !----------------------------------------------------------
  subroutine conv_DelGridQuant_2d(gq)
    real(dp),       pointer    :: gq(:,:)
    integer :: istat
    deallocate(gq,stat=istat)
  end subroutine conv_DelGridQuant_2d
  

  
  !----------------------------------------------------------------------
  subroutine conv_InitGridQuant_func(gd, gq, func)
    real(dp),         intent(inout) :: gq(0:)
    type(grid_def),   intent(in)    :: gd
    interface
       function func(x)
         use types; implicit none
         real(dp), intent(in) :: x
         real(dp)             :: func
       end function func
    end interface
    !-----------------------------------------
    integer :: i, istat, ny

    ny = assert_eq(gd%ny,ubound(gq,dim=1),"conv_InitGridQuant_func")
    do i = 0, ny
       gq(i) = func(i*gd%dy)
    end do
  end subroutine conv_InitGridQuant_func

  !----------------------------------------------------------------------
  subroutine conv_InitGridQuant_func2d(gd, gq, func)
    real(dp),         intent(inout) :: gq(0:,:)
    type(grid_def),   intent(in)    :: gd
    interface
       function func(x,n)
         use types; implicit none
         real(dp), intent(in) :: x
         integer , intent(in) :: n
         real(dp)             :: func(n)
       end function func
    end interface
    !-----------------------------------------
    integer :: i, ny, n

    ny = assert_eq(gd%ny,ubound(gq,dim=1),"conv_InitGridQuant_func")
    n  = ubound(gq, dim=2)
    do i = 0, ny
       gq(i,:) = func(i*gd%dy, n)
    end do
  end subroutine conv_InitGridQuant_func2d



  !--------------------------------------------------------------------
  function conv_EvalGridQuant(gd, gq, y) result(f)
    use interpolation
    type(grid_def), intent(in) :: gd
    real(dp), intent(in) :: gq(0:)
    real(dp), intent(in) :: y
    real(dp) :: f
    !-----------------------------------------
    integer, parameter :: npnt_min = 4, npnt_max = 10
    integer :: i, ny, npnt
    real(dp) :: ey, df
    real(dp), parameter :: resc_yvals(npnt_max) = (/ (i,i=0,npnt_max-1) /)

    ny = assert_eq(gd%ny,ubound(gq,dim=1),"conv_EvalGridQuant")
    npnt = min(npnt_max, max(npnt_min, gd%order))

    if (y > gd%ymax) then
       write(0,*) 'conv_EvalGridQuant: &
            &requested function value beyond maximum'
       write(0,*) 'y = ', y, 'ymax=',gd%ymax
       stop
    end if
    
    i = min(max(floor(y / gd%dy)-(npnt-1)/2,0),ny-npnt+1)
    call polint(resc_yvals(1:npnt),gq(i:i+npnt-1),y/gd%dy-i,f,df)
!!$    i = min(gd%ny - 1, floor(y / gd%dy))
!!$    ey = y/gd%dy - i
!!$    f  = (one-ey)*gq(i) + ey*gq(i+1)
  end function conv_EvalGridQuant
  

  !----------------------------------------------------------------------
  function conv_MomGridQuant(gd,gq,omega) result(res)
    type(grid_def), intent(in) :: gd
    real(dp), intent(in) :: gq(0:)
    real(dp), intent(in) :: omega
    real(dp) :: res
    !-----------------------------------------
    real(dp) :: weight, weightprod, dy
    integer :: i, ny

    ny = assert_eq(gd%ny,ubound(gq,dim=1),"conv_MomGridQuant")
    dy = gd%dy
    if (omega == zero) then
       weight = gd%dy
       weightprod = one
       res = half*weight*gq(0)
    else
       weightprod = exp(-dy*omega)
       weight = (exp(-dy*omega) - one + dy*omega)/(dy*omega**2)
       res = weight*gq(0)
       weight = weight + &
            & (exp(+dy*omega) - one - dy*omega)/(dy*omega**2)
    end if
    do i = 1, ny
       weight = weight * weightprod
       if (i == ny) weight = weight * half
       res = res + weight*gq(i)
    end do
  end function conv_MomGridQuant
  

  !----------------------------------------------------------------------
  subroutine conv_PrintGridQuant_1(gd,gq,dy,dev)
    type(grid_def), intent(in) :: gd
    real(dp), intent(in) :: gq(0:)
    real(dp), intent(in), optional :: dy
    integer,  intent(in), optional :: dev
    real(dp) :: dy_local, y, x, q
    integer :: ny, i, dev_local
    
    ny = assert_eq(gd%ny,ubound(gq,dim=1),'conv_PrintGridQuant')
    dy_local  = default_or_opt(gd%dy,dy)
    dev_local = default_or_opt(6,dev)

    ny = floor(gd%ymax / dy_local)
    do i = 0, ny
       y = i*dy_local
       q = conv_EvalGridQuant(gd, gq, y)
       write(dev_local,*) y, exp(-y),q
    end do
    
  end subroutine conv_PrintGridQuant_1

  !----------------------------------------------------------------------
  subroutine conv_PrintGridQuant_2(gd,gq,gq2,dy,dev)
    type(grid_def), intent(in) :: gd
    real(dp), intent(in) :: gq(0:),gq2(0:)
    real(dp), intent(in), optional :: dy
    integer,  intent(in), optional :: dev
    real(dp) :: dy_local, y, x, q,q2
    integer :: ny, i, dev_local
    
    ny = assert_eq(gd%ny,ubound(gq,dim=1),&
         & ubound(gq2,dim=1),'conv_PrintGridQuant')
    dy_local  = default_or_opt(gd%dy,dy)
    dev_local = default_or_opt(6,dev)

    ny = floor(gd%ymax / dy_local)
    do i = 0, ny
       y = i*dy_local
       q  = conv_EvalGridQuant(gd, gq, y)
       q2 = conv_EvalGridQuant(gd, gq2, y)
       write(dev_local,'(25es25.16)') y, exp(-y),q, q2
    end do
    
  end subroutine conv_PrintGridQuant_2

  !----------------------------------------------------------------------
  subroutine conv_PrintGridQuant_3(gd,gq,gq2,gq3,dy,dev)
    type(grid_def), intent(in) :: gd
    real(dp), intent(in) :: gq(0:),gq2(0:),gq3(0:)
    real(dp), intent(in), optional :: dy
    integer,  intent(in), optional :: dev
    real(dp) :: dy_local, y, x, q,q2,q3
    integer :: ny, i, dev_local
    
    ny = assert_eq(gd%ny,ubound(gq,dim=1),&
         & ubound(gq2,dim=1),ubound(gq3,dim=1),&
         & 'conv_PrintGridQuant: distributions must be same size')
    dy_local  = default_or_opt(gd%dy,dy)
    dev_local = default_or_opt(6,dev)

    ny = floor(gd%ymax / dy_local)
    do i = 0, ny
       y = i*dy_local
       q  = conv_EvalGridQuant(gd, gq, y)
       q2 = conv_EvalGridQuant(gd, gq2, y)
       q3 = conv_EvalGridQuant(gd, gq3, y)
       write(dev_local,'(25es25.16)') y, exp(-y),q, q2, q3
    end do
    
  end subroutine conv_PrintGridQuant_3
  
  !----------------------------------------------------------------------
  subroutine conv_PrintGridQuant_4(gd,gq,gq2,gq3,gq4,dy,dev)
    type(grid_def), intent(in) :: gd
    real(dp), intent(in) :: gq(0:),gq2(0:),gq3(0:),gq4(0:)
    real(dp), intent(in), optional :: dy
    integer,  intent(in), optional :: dev
    real(dp) :: dy_local, y, x, q,q2,q3, q4
    integer :: ny, i, dev_local
    
    ny = assert_eq(gd%ny,ubound(gq,dim=1),&
         & ubound(gq2,dim=1),ubound(gq3,dim=1),ubound(gq4,dim=1),&
         & 'conv_PrintGridQuant: distributions must be same size')
    dy_local  = default_or_opt(gd%dy,dy)
    dev_local = default_or_opt(6,dev)

    ny = floor(gd%ymax / dy_local)
    do i = 0, ny
       y = i*dy_local
       q  = conv_EvalGridQuant(gd, gq, y)
       q2 = conv_EvalGridQuant(gd, gq2, y)
       q3 = conv_EvalGridQuant(gd, gq3, y)
       q4 = conv_EvalGridQuant(gd, gq4, y)
       write(dev_local,'(25es25.16)') y, exp(-y),q, q2, q3, q4
    end do
    
  end subroutine conv_PrintGridQuant_4


  !======================================================================
  ! Stuff related to convolutions
  !
  subroutine conv_Seteps(eps_in)
    real(dp), intent(in) :: eps_in
    eps = eps_in
  end subroutine conv_Seteps


  subroutine conv_InitGridConv_gen(gd,gc)
    type(grid_def),  intent(in)    :: gd
    type(grid_conv), intent(inout) :: gc
    !-------------------------------------------
    integer :: istat
    
    gc%gd = gd
    ! this form of deallocate ought to be OK (?), but on lahey,
    ! compaq+condor (and perhaps elsewhere) it causes problems
    !deallocate(gc%conv,stat=istat)
    if (gd%order == LIN_ORDER) then
       allocate(gc%conv(0:gd%ny,2))
    else
       allocate(gc%conv(0:gd%ny,0:gd%order+1))
    end if
  end subroutine conv_InitGridConv_gen

  !--------------------------------------------------------------------
  ! initialise a grid convolution with zero
  subroutine conv_InitGridConv_zero(gd,gc)
    type(grid_def),  intent(in)    :: gd
    type(grid_conv), intent(inout) :: gc
    !-------------------------------------------
    
    call  conv_InitGridConv_gen(gd,gc)
    gc%conv = zero
  end subroutine conv_InitGridConv_zero

  !--------------------------------------------------------------------
  ! initialise a grid convolution with another gc, potentially multiplied
  ! by a factor
  subroutine conv_InitGridConv_gc(gc,gcin,fact)
    type(grid_conv), intent(inout) :: gc
    type(grid_conv), intent(in)    :: gcin
    real(dp),        intent(in), optional :: fact
    !-------------------------------------------

    call  conv_InitGridConv_gen(gcin%gd,gc)
    if (present(fact)) then
       gc%conv = gcin%conv * fact
    else
       gc%conv = gcin%conv
    end if
  end subroutine conv_InitGridConv_gc


  !----------------------------------------------------------------------
  ! Initialise a convoluter with the function to use in the 
  ! convolution. 
  subroutine conv_InitGridConv_func(gd,gc,func)
    type(grid_def),  intent(in)    :: gd
    type(grid_conv), intent(inout) :: gc
    interface
       function func(x)
         use types; implicit none
         real(dp), intent(in) :: x
         real(dp)             :: func
       end function func
    end interface
    !-------------------------------------------
    
    call conv_InitGridConv_zero(gd,gc)
    call conv_GridConvAdd(gc,func)
  end subroutine conv_InitGridConv_func

  !--------------------------------------------------------
  ! Remove memory associated with a given gc.
  subroutine conv_DelGridConv(gc)
    type(grid_conv), intent(inout) :: gc
    !-------------------------------------------
    integer :: istat
    
    deallocate(gc%conv,stat=istat)
    if (istat /= 0) then
       write(0,*) 'ERROR: problems with deallocation in conv_DelGridConv'
       write(0,*) one/sqrt(sin(zero))
       stop
    end if
  end subroutine conv_DelGridConv


  !--------------------------------------------------------------------
  ! Multiply the given GridConv by the relevant factor
  subroutine conv_MultGridConv(gc,fact)
    type(grid_conv), intent(inout) :: gc
    real(dp),        intent(in)    :: fact
    !-------------------------------------------
    gc%conv = gc%conv * fact
  end subroutine conv_MultGridConv

  

  !-------------------------------------------------------------
  ! To gc add a function for convolution
  subroutine conv_GridConvAdd_func(gc,func)
    use integrator; use convolution_communicator
    type(grid_conv), intent(inout), target :: gc
    interface
       function func(x)
         use types; implicit none
         real(dp), intent(in) :: x
         real(dp)             :: func
       end function func
    end interface
    !----------------------------------------------------
    real(dp), pointer :: dy
    integer,  pointer :: ny, order
    real(dp) :: upper,lower, yl,ym,yh
    integer  :: i,j, k
    !------------------------------------------------------
    real(dp) :: nodes(gc%gd%order+1), res
    integer  :: inode_one, il, ih, iy, jy, inode

    dy => gc%gd%dy
    ny => gc%gd%ny
    order => gc%gd%order

    cc_piece = cc_REAL
    if (gc%gd%order == LIN_ORDER) then
       do i = 1, ny
          ym = i*dy
          yl = ym - dy
          yh = ym + dy
          lower = ig_LinWeight(func,yl,ym,zero,one,eps) ! specify 0 at yl
          upper = ig_LinWeight(func,ym,yh,one,zero,eps) ! specify 0 at yh
          gc%conv(i,FULL) = gc%conv(i,FULL) + lower + upper
          gc%conv(i,UPPR) = gc%conv(i,UPPR) + upper
       end do
       !-- see CCN17-62 ----
       ym = zero
       yh = dy
       cc_piece = cc_REAL
       upper    = ig_LinWeight(func,ym,yh,zero,-one,eps)
       cc_piece = cc_REALVIRT
       upper    = upper + ig_LinWeight(func,ym,yh,one,one,eps) 
       cc_piece = cc_VIRT
       upper    = upper + ig_LinWeight(func,yh,-two*log(eps),one,one,eps)
       cc_piece = cc_DELTA
       upper    = upper + func(zero)
       gc%conv(0,FULL) = gc%conv(0,FULL) + upper
       gc%conv(0,UPPR) = gc%conv(0,UPPR) + upper
    else
       !-- CCN19 p.4 -------------
       ! index 0:          central points
       ! index 1..order+1: for dealing with last order+1 points
       !-- first do central points. Do an interval at a time
       !   i = 1, means interval from 0 to 1
       !-- the frist loop is missing something:: it should also be filling up
       !   some pieces in the second loop, at least in some cases
       do i = 1, ny
          !-- this is the range of interest
          yl = (i-1) * dy
          yh =  i    * dy
          !-- these are the interpolation points to be used
          il = max(0,i - (order+2)/2)
          ih = min(ny, il+order)
          il = ih - order
          !-- fill things up?
          nodes = (/ (j,j=il,ih) /) * dy
          do iy = il, ih
             res = conv_GCAf_Helper(func,yl,yh,il,iy,nodes)
             gc%conv(iy,0) = gc%conv(iy,0) + res
             !-- deal properly with special end array --
             !   try to be clever with limits, but there are no guarantees.
             !   It does however seem to work!
             do jy = max(i+order,iy), iy+order
                if (jy <= ny) then
                   gc%conv(jy,order+1-(jy-iy)) &
                        &= gc%conv(jy,order+1-(jy-iy)) + res
                end if
             end do
          end do
       end do
       !-- now deal with integrations close to end points
       ! k represents the end points
       do k = 1, ny
          if (k <= order) then
             yl = max(zero, (k-order)*dy)
             yh = k*dy
             !-- the interpolation points & yes, it is normal for them 
             !   to be negative
             !   it is related to the way the convolution is done for the case 
             !   i < order, i.e. effectively conv(0:order)*gq(order:0:-1)
             !   (actually conv(i, 1:order+1))
             ih = k
             il = k - order
             !-- fill things up
             nodes = (/ (j,j=il,ih) /) * dy
             do iy = il, ih
                res = conv_GCAf_Helper(func,yl,yh,il,iy,nodes)
                gc%conv(k,iy-il+1) = gc%conv(k,iy-il+1) + res
             end do
             cycle
          end if
          ! now do things region by region
          !-- i represents the region that we will study
          do i = max(1,k-order+1), k
             !-- this is the range of interest
             yl = (i-1) * dy
             yh =  i    * dy
             !-- these are the interpolation points to be used
             !   note extra min(k,...) compared to above formula
             il = max(0,i - (order+2)/2)
             ih = min(k,min(ny, il+order)) !-- could simplify expression
             il = ih - order
             nodes = (/ (j,j=il,ih) /) * dy
             do iy = max(il,k-order), ih
                res = conv_GCAf_Helper(func,yl,yh,il,iy,nodes)
                gc%conv(k,order+1-(k-iy)) = &
                     &gc%conv(k,order+1-(k-iy)) + res
             end do
          end do
       end do
       

!!$       do i = 1, ny
!!$          !-- the integration range
!!$          yl = max(zero, (i-order)*dy)
!!$          yh = i*dy
!!$          !-- the interpolation points & yes, it is normal for them 
!!$          !   to be negative
!!$          !   it is related to the way the convolution is done for the case 
!!$          !   i < order, i.e. effectively conv(0:order)*gq(order:0:-1)
!!$          !   (actually conv(i, 1:order+1))
!!$          ih = i
!!$          il = i - order
!!$          !-- fill things up
!!$          nodes = (/ (j,j=il,ih) /) * dy
!!$          do iy = il, ih
!!$             res = conv_GCAf_Helper(func,yl,yh,il,iy,nodes)
!!$             gc%conv(i,iy-il+1) = gc%conv(i,iy-il+1) + res
!!$          end do
!!$       end do
    end if
    
  end subroutine conv_GridConvAdd_func


  !---------------------------------------------------------------------
  ! look after some repetitive work...
  function conv_GCAf_Helper(func,yl,yh,il,iy,nodes) result(res)
    use integrator; use convolution_communicator
    interface
       function func(x)
         use types; implicit none
         real(dp), intent(in) :: x
         real(dp)             :: func
       end function func
    end interface
    real(dp), intent(in) :: yl, yh, nodes(:)
    integer,  intent(in) :: il, iy
    real(dp)             :: res
    integer :: inode
    inode = iy - il + 1
    res = zero
    if (yl == zero .and. iy == 0) then
       cc_piece = cc_VIRT
       res = res + ig_LinWeight(func, yh,-two*log(eps), one,one, eps)
       cc_piece = cc_DELTA
       res = res + func(zero)
       cc_piece = cc_REALVIRT
       res = res + ig_LinWeight(func, yl, yh, one, one, eps)
       cc_piece = cc_REAL
       res = res + ig_PolyWeight(func, yl, yh, nodes, inode, eps,wgtadd=-one)
    else
       cc_piece = cc_REAL
       res = ig_PolyWeight(func, yl, yh, nodes, inode, eps)
    end if
  end function conv_GCAf_Helper
  


  !-------------------------------------------------------------
  ! To gc add another grid convolution.
  subroutine conv_GridConvAdd_gc(gc,gcadd,fact)
    type(grid_conv), intent(inout) :: gc
    type(grid_conv), intent(in)  :: gcadd
    real(dp),        intent(in), optional  :: fact

    call conv_ValidateGD(gc%gd, gcadd%gd, 'conv_GridConvAdd_gc')
    if (present(fact)) then
       gc%conv = gc%conv + gcadd%conv * fact
    else
       gc%conv = gc%conv + gcadd%conv
    end if
  end subroutine conv_GridConvAdd_gc


  !--------------------------------------------------------------
  ! Carry out the convolution of gc on gq
  function conv_ConvQuant_scalar(gc,gq) result(gqout)
    type(grid_conv),  intent(in),target    :: gc
    real(dp),         intent(in)    :: gq(0:)
    real(dp)                        :: gqout(0:ubound(gq,dim=1))
    !---------------------------------------------
    integer :: i, ny, j
    integer, pointer :: order

    ny = assert_eq(gc%gd%ny,ubound(gq,dim=1),"conv_ConvQuant")
    order => gc%gd%order

    !-- Hopefully this will avoid some wasted convolutions?
    if (all(gq == zero)) then
       gqout = zero
       return
    end if
    

    if (order == LIN_ORDER) then
       !-- a test to avoid N^2 operations...
       if (all(gc%conv(:,FULL) == zero)) then
          gqout = zero
          return
       end if
       
       gqout(0) = zero
       do i = 1, ny
          !-- following is legal fortran but it is very slow sometimes
          gqout(i) =  sum(gq(0:i)*gc%conv(i:0:-1,FULL))
          !-- maybe the version that follows will be faster on some compilers
          !   on absoft it definitely is
!!$       gqout(i) = zero
!!$       do j = 0, i
!!$          gqout(i) = gqout(i) + gq(j)*gc%conv(i-j,FULL)
!!$       end do
          !write(0,*) i*gc%gd%dy,gqout(i), gc%conva(1)
          gqout(i) = (gqout(i)-gq(0)*gc%conv(i,UPPR))
       end do
    else
       gqout = zero
       !-- a test to avoid N^2 operations...
       if (all(gc%conv(:,0) == zero)) return
       !-- current writing is designed to reduce thrashing of cache. 
       !   It is not clear that it actually helps in any way
       !   Commented version theoretically has higher cache thrashing.
       do i = 1, ny
          if (i > order) then
             gqout(i) = sum(gc%conv(0:i-order-1,0)*gq(i:order+1:-1))
          end if
          !-- HIGH CACHE THRASH
          !gqout(i) = gqout(i) + sum(gc%conv(i,1:order+1)*gq(order:0:-1))
       end do
       !-- LOW CACHE THRASH
       do i = 1, order+1
          gqout(:) = gqout(:) + gc%conv(:,i)*gq(order+1-i)
       end do
    end if
    
  end function conv_ConvQuant_scalar



  !--------------------------------------------------------------
  ! Carry out the convolution of two convolution functions
  ! It is recommended that if one of them is singular at x=1, then this be 
  ! gca: this helps minimise the discretisation error.
  subroutine conv_ConvConv(gc,gca, gcb)
    type(grid_conv),  intent(inout) :: gc
    type(grid_conv),  intent(in)    :: gca, gcb
    !---------------------------------------------
    integer :: i, ny, j, order
    real(dp) :: deltafn(0:gca%gd%ny), res(0:gca%gd%ny)

    if (.not.(gca%gd == gcb%gd)) then
       write(0,*) 'Error: in conv_ConvConv, the two convolution functions are&
            & not on the same grid'
       stop
    else if (gca%gd%order == LIN_ORDER) then
       call  conv_InitGridConv_gen(gca%gd,gc)
       !-- this is just a guess. Now we will see if it is right
       gc%conv(:,FULL) = (gca .conv. gcb%conv(:,FULL)) +&
            &gca%conv(:,UPPR) * gcb%conv(0,FULL)
       
       !-- next 3 lines actually calculate LOWER. Then correct it to upper.
       !   Do the convolution of a deltafn by hand in order to
       !   be more efficient: tested that it gave identical answers
       deltafn = gcb%conv(:,FULL) - gcb%conv(:,UPPR)
       deltafn(0) = zero
       gc%conv(:,UPPR) = gca .conv. deltafn
       !gc%conv(:,UPPR) = gca .conv. (gcb .conv. deltafn)
       gc%conv(:,UPPR) = gc%conv(:,FULL) - gc%conv(:,UPPR)
    else if (gca%gd%order /= LIN_ORDER) then
       order = gca%gd%order
       call  conv_InitGridConv_gen(gca%gd,gc)
       !-- let us hope that the following works?
       !   in this first incarnation it will be roughly half the optimum
       !   speed because the inner convolution will be done "stupidly",
       !   using O(N^2) steps rather than the possible O(N)
       do j = 0, order+1
          !-- set up a delta function and its convolution
          deltafn = zero; deltafn(j) = one
          !-- try to avoid too much rubbish in the result?
          res = zero
          res = gca .conv. (gcb .conv. deltafn)
          !-- redistribute the result
          if (j /= order+1) then
             gc%conv(:,order+1-j) = res(:)
          else
             gc%conv(0:gca%gd%ny-(order+1),0) = res(order+1:gca%gd%ny)
             !-- The region conv(gca%gd%ny-order:) remains unfilled. This 
             !   is OK since in practice (current incarnation) it never 
             !   gets used.
          end if
       end do
       !-- now remember to test it!
    end if


  end subroutine conv_ConvConv





  !--------------------------------------------------------------
  ! Carry out the convolution of gc on gq: but not always the
  ! most efficient way of dealing with the problem...
  function conv_ConvQuant_mat(gc,gq) result(gqout)
    type(grid_conv),  intent(in)    :: gc(:,:)
    real(dp),         intent(in)    :: gq(0:,:)
    real(dp)                        :: gqout(0:ubound(gq,dim=1),size(gc,dim=1))
    !---------------------------------------------
    integer :: i, ny, ic, ir, ncol, nrow

    ny = assert_eq(gc(1,1)%gd%ny,ubound(gq,dim=1),"conv_ConvQuant")
    ncol = assert_eq(size(gc,dim=2),size(gq,dim=2),"conv_ConvQuant")
    nrow = size(gc,dim=1)

    gqout = zero
    do ir = 1, nrow
       do ic = 1, ncol
          !gqout(:,ir) = gqout(:,ir) + gc(ir,ic) .conv. gq(:,ic)
          gqout(:,ir) = gqout(:,ir) +&
               & conv_ConvQuant_scalar(gc(ir,ic), gq(:,ic))
       end do
    end do
  end function conv_ConvQuant_mat

  

  !======================================================================
  ! things for easier access to a given grid point
  ! (but may not be as fast as just calling the routine)
  function conv_gdval_gdv(gd,val) result(gdv)
    type(grid_def), intent(in) :: gd
    real(dp),       intent(in) :: val
    type(gdval) :: gdv
    gdv%gd  = gd
    gdv%val = val
  end function conv_gdval_gdv
  function conv_gdval_vgd(val,gd) result(gdv)
    type(grid_def), intent(in) :: gd
    real(dp),       intent(in) :: val
    type(gdval) :: gdv
    gdv%gd  = gd
    gdv%val = val
  end function conv_gdval_vgd
  function conv_EvalGridQuant_atx(gq, gdv) result(res)
    real(dp),    intent(in) :: gq(:)
    type(gdval), intent(in) :: gdv
    real(dp) :: res
    res = conv_EvalGridQuant(gdv%gd, gq, -log(gdv%val))
  end function conv_EvalGridQuant_atx
  
end module convolution
