!======================================================================
!
! This will provide routines such that an ordinary fortran 77 user or a 
! C user will be able to access the resummation using for the fixed
! order info a file generated by dis-shapes-mc.
!
! Contrary to what happens with other modules routines here should
! have normal f77 interfaces, and be accessible without an explicit
! interface. 
!
! For f90 users it may be convenient to "use driver_withdsm".
!
! $Id: driver_withdsm.f90,v 1.6 2001/04/23 13:54:21 salam Exp $




!====================================================================
! Call this to start the program
! Reads in dsm file. Sets up colour factors. Extracts an array
! of fixed_order_evshp corresponding to a single event shape.
subroutine dwd_start(dsm_file, evshp_id, FOE_handle_lo, FOE_handle_hi)
  use driver_withdsm_hidden
  implicit none
  character(len=*), intent(in)  :: dsm_file
  integer,          intent(in)  :: evshp_id
  integer,          intent(out) :: FOE_handle_lo, FOE_handle_hi

  call dwdh_start(dsm_file, evshp_id, FOE_handle_lo, FOE_handle_hi)
end subroutine dwd_start

!======================================================================
! Returns some basic info about the fixed_order_evshp corresponding to
! a given handle
subroutine dwd_info(FOE_handle, xbj, ybj, Q, evshp_id)
  use driver_withdsm_hidden
  use types; implicit none
  integer,  intent(in)  :: FOE_handle
  real(dp), intent(out) :: xbj, ybj, Q
  integer,  intent(out) :: evshp_id

  call dwdh_info(FOE_handle, xbj, ybj, Q, evshp_id)
end subroutine dwd_info
!======================================================================
! Returns some extra info about the fixed_order_evshp corresponding to
! a given handle; try a new tack: write this routine just once...
subroutine dwd_extrainfo(FOE_handle, pdf_id, muF_Q, FactScheme, nf_lcl)
  use driver_withdsm_hidden; use fo_evshp; use handle_classes; use qcd
  use types; implicit none
  integer,  intent(in)  :: FOE_handle
  real(dp), intent(out) :: muF_Q
  integer,  intent(out) :: pdf_id, FactScheme, nf_lcl
  !-------------------------------------------------
  integer :: i
  i = hc_ElementId(FOE_handle, hc_FOEvshp)
  muF_Q   = evarr(i)%muF_Q
  pdf_id = evarr(i)%pdf_id
  nf_lcl = nf_int  ! taken from "qcd"
  FactScheme = set%factscheme
end subroutine dwd_extrainfo


!======================================================================
! Given a fixed_order_evshp handle and a set of nbin event-shape values 
! (varvals) it returns the integrated cross section at those event shape
! values. 
!
! The various symbolic constants  MatchType, EvlnType, PowerType
! can be obtained by placing the following line at the beginning of a 
! f77 subprogram and compiling with f90:
! 
!   "use dwd_withdsm"
!
subroutine dwd_dist(FOE_handle, nbin, varvals, sigmavals, alfas,  muR_Q, &
     &               alfa0, muI, MatchType, EvlnType, PowerType)
  use driver_withdsm_hidden
  use types; implicit none
  integer,  intent(in)  :: FOE_handle
  integer,  intent(in)  :: nbin
  real(dp), intent(in)  :: varvals(*)
  real(dp), intent(out) :: sigmavals(*)
  real(dp), intent(in)  :: alfas, muR_Q, alfa0, muI
  integer,  intent(in)  :: MatchType, EvlnType, PowerType
  call dwdh_dist(FOE_handle, varvals(1:nbin), sigmavals(1:nbin), &
       &alfas,  muR_Q, alfa0, muI, MatchType, EvlnType, PowerType)
end subroutine dwd_dist



!====================================================================
! Things that the users may (ought to) have access to if they are f90.
!
! This includes a number of constants.
!
module driver_withdsm
  use resummation_definitions
  use event_shape_ids
  !-- give people these interfaces too!
  use driver_generic_interfaces
  implicit none

  interface
     subroutine dwd_start(dsm_file, evshp_id, FOE_handle_lo, FOE_handle_hi)
       implicit none
       character(len=*), intent(in)  :: dsm_file
       integer,          intent(in)  :: evshp_id
       integer,          intent(out) :: FOE_handle_lo, FOE_handle_hi
     end subroutine dwd_start
  end interface
  
  interface
     subroutine dwd_info(FOE_handle, xbj, ybj, Q, evshp_id)
       use types; implicit none
       integer,  intent(in)  :: FOE_handle
       real(dp), intent(out) :: xbj, ybj, Q
       integer,  intent(out) :: evshp_id
     end subroutine dwd_info
  end interface
  
  interface
     subroutine dwd_extrainfo(FOE_handle, pdf_id, muF_Q, FactScheme, nf_lcl)
       use types; implicit none
       integer,  intent(in)  :: FOE_handle
       real(dp), intent(out) :: muF_Q
       integer,  intent(out) :: pdf_id, FactScheme, nf_lcl
     end subroutine dwd_extrainfo
  end interface
  
  interface
     subroutine dwd_dist(FOE_handle, nbin, varvals, sigmavals, alfas,  muR_Q, &
          &               alfa0, muI, MatchType, EvlnType, PowerType)
       use types; implicit none
       integer,  intent(in)  :: FOE_handle
       integer,  intent(in)  :: nbin
       real(dp), intent(in)  :: varvals(*)
       real(dp), intent(out) :: sigmavals(*)
       real(dp), intent(in)  :: alfas, muR_Q, alfa0, muI
       integer,  intent(in)  :: MatchType, EvlnType, PowerType
     end subroutine dwd_dist
  end interface

end module driver_withdsm

