!====================================================================
! This will contain routines that the users are best-off not 
! having direct access to normally. These are the routines that do the work
! for driver_withdsm. 
!
! $Id: driver_withdsm_hidden.f90,v 1.6 2002/05/23 14:40:06 salam Exp $
module driver_withdsm_hidden
  use types; use consts_dp; use qcd
  use warnings_and_errors
  use parser; use fo_evshp; use event_shapes
  use handle_classes; use resummation_manager
  use matching_cache
  implicit none
  private

  type(match_cache) :: mcache
  !-- these two can be accessed from driver_withdsm
  !   (Yes, it is ugly to have them public, but it avoids doubling a certain
  !    number of subroutine calls)...
  type(fixed_order_set),            public  :: set
  type(fixed_order_evshp), pointer, public :: evarr(:)
  logical :: started = .false.

  public :: dwdh_start, dwdh_info, dwdh_dist

  !-- not really for public consumption, but useful for me...
  public :: dwdh_dist_ash, dwdh_mean_ash

contains
  !------------------------------------------------------------------------
  ! May only be called once!
  ! Starts things up for a given event shape.
  subroutine dwdh_start(dsm_file, evshp_id, FOE_handle_lo, FOE_handle_hi)
    character(len=*), intent(in)  :: dsm_file
    integer,          intent(in)  :: evshp_id
    integer,          intent(out) :: FOE_handle_lo, FOE_handle_hi
    !---------------------------------------------------
    integer  :: idev, iostat
    real(dp) :: xmin

    if (started) then
       call wae_error('dwdh_start','This routine may only be called &
            &once during program execution')
    else
       started = .true.
    end if
    

    idev = dwdh_GetNewDevice()
    write(0,*) 'Will attempt to open '//trim(dsm_file)//&
         &' for fixed order results'
    open(unit=idev,file=trim(dsm_file),status='OLD',iostat=iostat)
    if (iostat/=0) call wae_error('dwdh_start','Could not open dsm_file')
    
    !-- initialise a set --------------------
    call parser_ReadFile(idev, set)
    call qcd_SetNf(set%nf)
    call qcd_SetGroup(set%ca, set%cf, set%tr)

    !-- turn the set into an array with just our event shape
    !   Keep all x and Q values
    call foe_ListToArray(set%first, evarr, es_ids=(/evshp_id/))
    !-- set up matching cache --
    xmin = minval(evarr%xbj)
    call mca_InitCache(mcache, xmin, dy=0.05_dp, factscheme=set%factscheme)
    call foe_SetupMH(mcache, evarr)

    FOE_handle_lo = hc_handle(lbound(evarr,dim=1), hc_FOEvshp)
    FOE_handle_hi = hc_handle(ubound(evarr,dim=1), hc_FOEvshp)
  end subroutine dwdh_start
  

  !-------------------------------------------------------------------
  ! Returns basic info about a fixed-order event-shape
  subroutine dwdh_info(FOE_handle, xbj, ybj, Q, evshp_id)
    integer,  intent(in)  :: FOE_handle
    real(dp), intent(out) :: xbj, ybj, Q
    integer,  intent(out) :: evshp_id
    !-----------------------------------
    integer :: i
    i = hc_ElementId(FOE_handle, hc_FOEvshp)
    xbj      = evarr(i)%xbj
    ybj      = evarr(i)%ybj
    Q        = evarr(i)%Q
    evshp_id = evarr(i)%evshp_id
  end subroutine dwdh_info
  

  !------------------------------------------------------------------
  ! Generate a distribution
  subroutine dwdh_dist(FOE_handle, varvals, sigmavals, alfas,  muR_Q, &
       &               alfa0, muI, MatchType, EvlnType, PowerType)
    integer,  intent(in)  :: FOE_handle
    real(dp), intent(in)  :: varvals(:)
    real(dp), intent(out) :: sigmavals(:)
    real(dp), intent(in)  :: alfas, muR_Q, alfa0, muI
    integer,  intent(in)  :: MatchType, EvlnType, PowerType
    !--- 
    integer :: i

    i = hc_ElementId(FOE_handle, hc_FOEvshp)
    call rm_FOEvshpResum(evarr(i), varvals,sigmavals,alfas=alfas,&
         &muR_Q = muR_Q, MatchType=MatchType, EvlnType=EvlnType,&
         & PowerType = PowerType, alfa0=alfa0, muI=muI)
  end subroutine dwdh_dist


  !------------------------------------------------------------------
  ! Generate a distribution using an ash
  subroutine dwdh_dist_ash(FOE_handle, varvals, sigmavals, ash,  muR_Q, &
       &               alfa0, muI, MatchType, EvlnType, PowerType)
    use alfas_server
    integer,  intent(in)  :: FOE_handle
    real(dp), intent(in)  :: varvals(:)
    real(dp), intent(out) :: sigmavals(:)
    type(as_handle), intent(in) :: ash
    real(dp), intent(in)  :: muR_Q, alfa0, muI
    integer,  intent(in)  :: MatchType, EvlnType, PowerType
    !--- 
    integer :: i

    i = hc_ElementId(FOE_handle, hc_FOEvshp)
    call rm_FOEvshpResum(evarr(i), varvals, sigmavals, ash=ash,&
         &muR_Q = muR_Q, MatchType=MatchType, EvlnType=EvlnType,&
         & PowerType = PowerType, alfa0=alfa0, muI=muI)
  end subroutine dwdh_dist_ash
  

  !------------------------------------------------------------------
  ! return a mean value
  function dwdh_mean_ash(FOE_handle, ash,  muR_Q, alfa0, muI, PowerType)&
       &       result(mean)
    use alfas_server; use means_manager
    real(dp)                    :: mean
    integer,  intent(in)        :: FOE_handle
    type(as_handle), intent(in) :: ash
    real(dp), intent(in)        :: muR_Q, alfa0, muI
    integer,  intent(in)        :: PowerType
    !--- 
    integer :: i

    i = hc_ElementId(FOE_handle, hc_FOEvshp)
!!$    mean = zero
    mean = mm_MeanPrediction(evarr(i), ash, muR_Q, PowerType, alfa0, muI)
  end function dwdh_mean_ash
  


  !--------------------------------------------------------------
  ! NOT an appropriate place for this!
  function dwdh_GetNewDevice() result(dev)
    integer :: dev
    !--------------------------------------------------------------
    integer, parameter :: dev_low = 30
    logical :: exist, opened
    integer :: iostat
    character(len=*), parameter :: subname='dwdh_GetNewDevice'
    dev = dev_low
    do
       inquire(unit=dev,iostat=iostat, exist=exist, opened=opened)
       if (iostat /= 0) call wae_error(subname, 'iostat non zero')
       if (.not. exist) call wae_error(subname, 'available devices exhausted')
       if (.not. opened) exit
       dev = dev + 1 
    end do
  end function dwdh_GetNewDevice

end module driver_withdsm_hidden


