!======================================================================
! Things relating to fixed order event shape distributions
! Currently there is a lot of overlap between parser and fo_evshp.
! But in general, there are some routines in fo_evshp which really
! have nothing whatsoever to do with parser
!
! $Id: fo_evshp.f90,v 1.4 2001/04/23 13:54:22 salam Exp $
module fo_evshp
  use types; use consts_dp
  use matching !-- needed for the definition of match_holder
  use holders  !-- needed for other definitions
  implicit none
  private
  

  type fixed_order_evshp
     integer  :: pdf_id, evshp_id
     real(dp) :: muF_Q, xbj, Q, ybj, Elim, ESlim
     real(dp) :: sigma(2), sigerr(2), sigElim(2), sigESlim(2), mean(2) 
     ! (nbin,3): put nbin first so that interpolation
     !            does not require a copy of the array
     real(dp), pointer :: dist1(:,:), dist2(:,:)
     ! integrated dists: nbin+1
     real(dp), pointer :: disti1(:,:), disti2(:,:)
     ! will be set up later...
     type(match_holder),      pointer :: mh
     type(sigma_holder),      pointer :: sh
     type(eventshape_holder), pointer :: esh
     real(dp),                pointer :: pdf(:,:)
  end type fixed_order_evshp
  
  !-- element of a linked list ---------
  !   end of list is indicated by such an item with evshp nullified.
  type fixed_order_item
     type(fixed_order_evshp), pointer :: evshp
     type(fixed_order_item),  pointer :: next
  end type fixed_order_item
  

  type fixed_order_set
     real(dp) :: ca, cf, tr
     integer  :: nf, factscheme
     !-- start of linked list ---------------------------
     type(fixed_order_item), pointer :: first
  end type fixed_order_set
  

  public :: fixed_order_evshp, fixed_order_item, fixed_order_set
  public :: foe_ListToArray, foe_DistIntegrated

  interface foe_SetupMH
     module procedure foe_SetupMH_1, foe_SetupMH_n
  end interface
  public :: foe_SetupMH


contains
  !-----------------------------------------------------------------
  ! takes the first element of a list, and converts it to an array of
  ! fixed_order_evshp. This involves copying a certain amount of
  ! information, but the result is easier to use in general... Original
  ! list is kept intact
  subroutine foe_ListToArray(first, array, es_ids, pdf_ids, Qmin, Qmax)
    type(fixed_order_item),  pointer :: first
    type(fixed_order_evshp), pointer :: array(:)
    integer, intent(in), optional       :: es_ids(:)
    integer, intent(in), optional       :: pdf_ids(:)
    real(dp), intent(in), optional :: Qmin, Qmax
    ! IF ADD OPTIONS: REMEMBER THEY MUST BE CHANGED IN both CALLS
    ! TO foe_IsTrue AND ALSO in that subroutine itself
    !--------------------------------------------
    type(fixed_order_item), pointer :: this
    type(fixed_order_evshp), pointer :: evshp
    integer :: i, n
    
    n = 0
    if (associated(first)) then
       this => first
       do
          !-- 1st way of ending the list -------
          if (.not. associated(this%evshp)) exit
          if (foe_IsTrue(this%evshp,es_ids, pdf_ids,Qmin,Qmax)) n = n + 1
          !-- 2nd way of ending the list -------
          if (.not. associated(this%next)) exit
          this => this%next
       end do
    end if

    if (n == 0) then
       write(0,*) 'WARNING in foe_ListToArray: no event shapes were&
            & selected' 
    end if

    allocate(array(n))
    this => first
    i = 0
    do 
       evshp => this%evshp
       if (foe_IsTrue(this%evshp,es_ids, pdf_ids, Qmin,Qmax)) then
          i = i + 1
          array(i) = this%evshp
       end if
       if (i == n) exit
       this => this%next
    end do
  end subroutine foe_ListToArray
  

  !----------------------------------------------------------------------
  ! If none ofthe optional arguments is present, return true
  ! Otherwise, return true only if we are in the list of es_ids, pdf_ids,
  ! and any other extra limits
  function foe_IsTrue(evshp, es_ids, pdf_ids,Qmin,Qmax) result(IsTrue)
    logical                             :: IsTrue
    type(fixed_order_evshp), intent(in) :: evshp
    integer, intent(in), optional       :: es_ids(:)
    integer, intent(in), optional       :: pdf_ids(:)
    real(dp), intent(in), optional      :: Qmin,Qmax
    !------------------------------------------------------------

    IsTrue = .true.
    if (present(es_ids)) then
       IsTrue = IsTrue .and. any(es_ids == evshp%evshp_id)
    end if
    if (present(pdf_ids)) then
       IsTrue = IsTrue .and. any(pdf_ids == evshp%pdf_id)
    end if
    if (present(Qmin)) then
       IsTrue = IsTrue .and. (evshp%Q >= Qmin)
    end if
    if (present(Qmax)) then
       IsTrue = IsTrue .and. (evshp%Q <= Qmax)
    end if
  end function foe_IsTrue
  
  !----------------------------------------------------------------------
  ! Set up the mh, and also set up the integrated distribution
  subroutine foe_SetupMH_1(cache, evshp)
    use matching_cache
    type(match_cache),       intent(inout) :: cache
    type(fixed_order_evshp), intent(inout) :: evshp

    call mca_GetMH(cache, evshp%mh, evshp%sh, evshp%esh, evshp%pdf, &
         & evshp%evshp_id, evshp%pdf_id, &
         &evshp%xbj, evshp%ybj, muF_Q=evshp%muF_Q, Q=evshp%Q)
    call foe_SetupIntegrated_1(evshp)
  end subroutine foe_SetupMH_1
  subroutine foe_SetupMH_n(cache, evshp)
    use matching_cache
    type(match_cache),       intent(inout) :: cache
    type(fixed_order_evshp), intent(inout) :: evshp(:)
    integer :: i
    do i = 1, size(evshp)
       call mca_GetMH(cache, evshp(i)%mh, evshp(i)%sh, evshp(i)%esh, &
            & evshp(i)%pdf, evshp(i)%evshp_id, evshp(i)%pdf_id,&
            & evshp(i)%xbj, evshp(i)%ybj, muF_Q=evshp(i)%muF_Q, Q=evshp(i)%Q)
       call foe_SetupIntegrated_1(evshp(i))
    end do
    
  end subroutine foe_SetupMH_n
  

  !----------------------------------------------------------------------
  ! makes use of various bits of information to set up the correct
  ! integrated distribution. Currently does not support event-shapes like
  ! the C-parameter which have divergences in middle; nor does it support
  ! event shapes which go beyond the upper limit of the array
  subroutine foe_SetupIntegrated_1(evshp)
    use warnings_and_errors; use event_shapes
    type(fixed_order_evshp), intent(inout) :: evshp
    integer, parameter :: max_warn = 1
    integer, save      :: warn_id = warn_id_INIT
    character(len=*), parameter :: warnmesg="foe_SetupIntegrated_1: &
         &completing a partial distribution with sigESlim info"
    integer :: nbin

    if (.not. associated(evshp%mh)) call wae_error(&
         &'foe_SetupIntegrated: requires mh to have been set up' )

    call foe_integrate_dist(evshp%dist1, evshp%disti1, evshp%mh%F2L_1 -&
         & evshp%sigElim(1))
    !call foe_integrate_dist(evshp%dist1, evshp%disti1, evshp%mh%F2L_1)
    call foe_integrate_dist(evshp%dist2, evshp%disti2, &
         & -evshp%sigElim(2))

    !-- for each order work out whether we need to complete things 
    !   above ESlim
    nbin = ubound(evshp%disti1,dim=1)
    if (abs(evshp%ESlim - evshp%disti1(nbin,1)) < 1e-5_dp) then
       evshp%disti1(:,2) = evshp%disti1(:,2) - evshp%sigESlim(1)
       call wae_warn(max_warn, warn_id, warnmesg)
    end if
    nbin = ubound(evshp%disti2,dim=1)
    if (abs(evshp%ESlim - evshp%disti2(nbin,1)) < 1e-5_dp) then
       evshp%disti2(:,2) = evshp%disti2(:,2) - evshp%sigESlim(2)
       call wae_warn(max_warn, warn_id, warnmesg)
    end if
    
  end subroutine foe_SetupIntegrated_1

!!$  ! same, but for an array
!!$  subroutine foe_SetupIntegrated_n(evshp)
!!$    type(fixed_order_evshp), intent(inout) :: evshp(:)
!!$    integer :: i
!!$    do i = 1, size(evshp)
!!$       call foe_SetupIntegrated_1(evshp(i))
!!$    end do
!!$  end subroutine foe_SetupIntegrated_n
  

  !---------------------------------------------------------------
  ! not the most sophisticated
  subroutine foe_integrate_dist(dist,itgd, upval)
    use warnings_and_errors
    real(dp), intent(in) :: dist(:,:), upval
    real(dp), pointer    :: itgd(:,:)
    integer  :: nbin, i
    real(dp) :: dvar
    integer, save      :: warn_id_exp = 0, warn_id_log = 0
    integer, parameter :: max_warn = 4

    nbin = size(dist,dim=1)
    allocate(itgd(0:nbin,4))

    dvar = dist(nbin,1) - dist(nbin-1,1)
    if (dvar <= 0) then
       write(0,*) 'ERROR in foe_integrate_dist: &
            &Bins are expected to be increasing' 
       stop
    end if
    itgd(nbin,:3) = (/ dist(nbin,1)+half*dvar, upval, zero /)
    do i = nbin-1, 0, -1
       itgd(i,1) = itgd(i+1, 1) - dvar
       itgd(i,2) = itgd(i+1, 2) - dvar*dist(i+1, 2)
       itgd(i,3) = sqrt(itgd(i+1, 3)**2 + (dvar*dist(i+1, 3))**2)
    end do
    !-- use as signal for logarithmic binning, the fact that any of the
    !   non-external bin edges be negative
    !-- In such a case convert log coordinates to linear coords
    if (any(itgd(1:nbin-1,1) < zero)) then
       itgd(:,4) = -itgd(:,1)
       itgd(:,1) = exp(itgd(:,1))
       call wae_warn(max_warn, warn_id_exp, &
            &'foe_integrate_dist: Converted from log(var) to var')
!!$       if (nwarn < nwarn_limit) then
!!$          write(0,*) 'Converted from log(var) to var'
!!$          nwarn = nwarn + 1
!!$          if (nwarn == nwarn_limit) write(0,*) '      no more such notices&
!!$               & will be issued'
!!$       end if
    else
       !-- set up also the log of the variable. NB log(1/var)!
       where (itgd(:,1) >= zero)       
          itgd(:,4) = -log(itgd(:,1))
       end where
       if (itgd(0,1) <= 1e-7_dp) then
          itgd(0,4) = itgd(1,4) * two
          call wae_warn(max_warn, warn_id_log, &
               &'foe_integrate_dist: fudged log(1/var) in lowest bin')
       end if
       
    end if
    
    
  end subroutine foe_integrate_dist

  !---------------------------------------------------------------
  ! Return the value of the LO and NLO integrated distribution
  ! as worked out by interpolation.
  !
  ! This should be the O(as) and O(as^2) contributions to the
  ! the cross section (normalised to LO) of the event-shape being
  ! below var?
  !
  subroutine foe_DistIntegrated(evshp, var, disti1, disti2)
    use interpolation
    type(fixed_order_evshp), intent(in)  :: evshp
    real(dp),                intent(in)  :: var
    real(dp),                intent(out) :: disti1, disti2
    !---------------------------------
    integer  :: npnt = 4, n
    real(dp) :: err1, err2, lnvar

    if (   .not. associated(evshp%disti1) .or. &
         & .not. associated(evshp%disti2)) then
       write(0,*) 'Error in foe_DistIntegrated:'
       write(0,*) 'Integrated distributions are not present.'
    end if
    
    lnvar = -log(var)

    n = ubound(evshp%disti1,dim=1)
    if (var < evshp%disti1(n,1)) then
       call interpolate(evshp%disti1(:,4), evshp%disti1(:,2), &
            &                           lnvar, disti1, err1, npnt)
    else
       disti1 = evshp%disti1(n,2); err1 = zero
    end if


    n = ubound(evshp%disti2,dim=1)
    if (var < evshp%disti2(n,1)) then
       call interpolate(evshp%disti2(:,4), evshp%disti2(:,2), &
            &                           lnvar, disti2, err1, npnt)
    else
       disti2 = evshp%disti2(n,2); err1 = zero
    end if
    
!!$    call interpolate(evshp%disti2(:,4), evshp%disti2(:,2), &
!!$         &                                lnvar, disti2, err2, npnt)
    !write(0,*) var, disti1, err1

  end subroutine foe_DistIntegrated
  


end module fo_evshp
