!======================================================================
! Program for merging two MC files. For the time being require that 
! they come from the same generator...
!
! Actually the simplest thing is perhaps to avoid almost all checks:
! just do something automatic: if there is a +- then the numbers need
! to be added in quadrature (minimising the error).
!
! If there is an NEV: add the numbers?
!
! $Id: join-ybj2s.f90,v 1.4 2001/11/01 18:59:04 salam Exp $
!======================================================================


module joinybj2s_helper
  use types; use consts_dp
  use join_genhelper
  use sub_defs_io
  use warnings_and_errors
  implicit none
  private

  public :: join2_doit


contains

  subroutine join2_doit
    integer, parameter :: max_nfiles = 2
    integer, parameter :: nfiles = 2, maxlines_for_y = 30
    integer  :: i, idev(max_nfiles), j, iostat, linenumb, ix, idv
    
    character(len=*), parameter :: a='(a)'
    character(len=150)          :: oneline, filename
    character(len=150)  :: line(nfiles)
    integer             :: ltype(nfiles)
    real(dp)            :: vals(3,nfiles), ybj_vals(nfiles), weights(nfiles)
    real(dp)            :: rts, ybj_wanted, xbj, Qval

    real(dp) :: sigElim1_exact, sigElim1(2)

    real(dp) :: combres(2)
    character(len=*), parameter :: format_mean = &
         &'(a,"= ",es22.12," +- ",es22.12)'

    if (lcl_iargc() /= 3) then
       call wae_error('join-ybj2s: Usage should be',&
            &'join-ybj2s sqrt(s) file1 file2' )
    end if
    

    !--- TESTING ---
!!$    ybj_wanted  = dble_val_arg(1)
!!$    ybj_vals(1) = dble_val_arg(2)
!!$    ybj_vals(2) = dble_val_arg(3)
!!$    call set_ybj_weights(ybj_wanted, ybj_vals, weights)
!!$    ybj_vals = - ybj_vals**2/(1+(1-ybj_vals)**2)
!!$    ybj_wanted = - ybj_wanted**2/(1+(1-ybj_wanted)**2)
!!$    write(0,*) sum(weights * ybj_vals), ybj_wanted
!!$    stop

    rts     = dble_val_arg(1)
    idev(1) = idev_open_arg(2)
    idev(2) = idev_open_arg(3)




    linenumb = 1
    !-- n lines of rubbish which need not necessarily be the same in each file
    !   stop when reach colour factors
    do i = 1, nfiles
       do j = 1, maxlines_for_y
          read(idev(i),a) line(i)
          !---- DO NOT CHECK FOR INCOMPATIBILITY BETWEEN COLOUR FACTORS?
          !     IS THIS NOT A BIT DANGEROUS?
          !-- stop when reach ybj indicator factors
          if (index(line(i),'ybj =') > 0) then
             ybj_vals(i) = label_val(idev(i), line(i), 'ybj =')
             if (i == 1) write(6,'(a,es20.14)') '( rts = ',rts
             exit
          end if
          if (i == 1) then
             if (j == 1) then
                write(6,a) trim(line(i))//' fixed-s results'
             else
                write(6,a) trim(line(i))
             end if
             linenumb = linenumb + 1
          end if
       end do
       if (j > maxlines_for_y) call wae_error('join2_doit:',&
            & 'Could not find specification of ybj value in file '//trim(num2char(i)))
    end do


    outer: do 
       linenumb = linenumb + 1
       do i = 1, nfiles
          read(idev(i),a,iostat=iostat) line(i)
          if (iostat /= 0) exit outer
          call SplitIt(line(i),ltype(i),vals(:,i))
       end do
       
       if (any(ltype(2:) /= ltype(1))) then
          write(0,*) 'Incompatibility of files around line', linenumb
          stop
       end if
       
       select case(ltype(1))
       case(ltype_nop)
          write(6,a) trim(line(1))
       case(ltype_val, ltype_esname, ltype_pdfname)
          if (any(abs(vals(1,2:) - vals(1,1)) > &
               &abs(vals(1,1))*1e-5_dp)) then 
             write(0,*) 'Incompatibility of files around line', linenumb
             stop
          end if
          write(6,a) trim(line(1))
          !-- we need to check if xbj and Qval are being redefined
          !   here
          if (ltype(1) == ltype_val) then
             if (line(1)(1:5) == "( x =") &
                  &xbj = label_val_line(line(1),"x =")
             if (line(1)(1:5) == "( Q =") then
                Qval = label_val_line(line(1),"Q =")
                ybj_wanted = (Qval/rts)**2 / xbj
                call set_ybj_weights(ybj_wanted, ybj_vals, weights)
                !write(0,*) '------------------------------------------------'
                !write(0,*) 'x and Q are', xbj, Qval
                !write(0,*) 'New ybj is', ybj_wanted
                !write(0,*) 'weights are ', weights
             end if
          end if
       case(ltype_valerr)
          combres = comb_weights(vals(1:2,:), weights)
          ix = index(line(1),'=')
          write(6,format_mean) line(1)(1:ix-1),combres(1), combres(2)
       case(ltype_3num)
          if (any(abs(vals(1,2:) - vals(1,1)) > &
               &abs(vals(1,1))*1e-5_dp)) then 
             write(0,*) 'Incompatibility of files around line', linenumb
             stop
          end if
          write(6,'(3es25.15)') vals(1,1), comb_weights(vals(2:3,:),weights)
       end select
    end do outer

!=============== STOPPED WORK HERE ===================================
!            TEST                   IT!
!=====================================================================
  end subroutine join2_doit
  

  !--------------------------------------------------------------
  ! Determine weights needed for two ybj values in order to get 
  ! the desired ybj value.
  !
  ! It is organised assuming that quantities are normalised relative 
  ! to the leading order cross section. Accordingly the weights have 
  ! to add up to 1, since the LO F2 part is already correct (and equal)
  ! for both ybj values.
  subroutine set_ybj_weights(ybj_wanted, ybj_vals, weights)
    real(dp), intent(in)  :: ybj_wanted, ybj_vals(2)
    real(dp), intent(out) :: weights(2)
    !-----------------------------------
    real(dp) :: FL_wanted, FL_in(2)

    FL_wanted = - ybj_wanted**2/(1+(1-ybj_wanted)**2)
    FL_in     = - ybj_vals**2/(1+(1-ybj_vals)**2)
    
    !-- now just "linear interpolation"
    weights(1) = (FL_wanted - FL_in(2)) / (FL_in(1) - FL_in(2))
    weights(2) = one - weights(1)
  end subroutine set_ybj_weights
  
end module joinybj2s_helper



program join_renorm
  use joinybj2s_helper
  implicit none

  call join2_doit

end program join_renorm
