!======================================================================
! General helper things for all joining routines
!
! NB: currently this routine does not follow the required coding standards
!     since public entities do not have any explicit prefix.
!
! $Id: join_genhelper.f90,v 1.6 2001/10/24 12:52:52 salam Exp $
module join_genhelper
  use types; use consts_dp
  implicit none
  private

  integer, parameter, public :: ltype_nop     = 1
  integer, parameter, public :: ltype_val     = 2
  integer, parameter, public :: ltype_valerr  = 3
  integer, parameter, public :: ltype_3num    = 4
  integer, parameter, public :: ltype_esname  = 5
  integer, parameter, public :: ltype_pdfname = 6

  public :: comb, comb_weights, SplitIt, label_val, label_val_PDF, label_val_line

contains
  
  
  function comb(vals)
    real(dp)  :: comb(2)
    real(dp), intent(in)  :: vals(:,:)
    real(dp)  :: weights(size(vals,dim=2)), sumweights

    where(vals(2,:) /= zero)
       weights = one / (vals(2,:)**2)
    elsewhere
       weights = zero
    end where

    sumweights = sum(weights)
    if (sumweights /= zero) weights = weights / sumweights
    
    comb(1) = sum(weights * vals(1,:))
    comb(2) = sqrt(sum((vals(2,:) * weights)**2))
  end function comb



  !-----------------------------------------------
  ! Should be correct for any weights one likes
  function comb_weights(vals, weights) result(comb)
    real(dp)  :: comb(2)
    real(dp), intent(in)  :: vals(:,:), weights(:)

    comb(1) = sum(weights * vals(1,:))
    comb(2) = sqrt(sum((vals(2,:) * weights)**2))
  end function comb_weights

  


  !------------------------------------------------------------------
  ! decides what kind of line this is and splits it up to some extent
  subroutine SplitIt(line, ltype, vals)
    use event_shapes; use pdf_names
    character(len=*), intent(in) :: line
    integer,  intent(out)        :: ltype
    real(dp), intent(out)        :: vals(3)
    character(len=*), parameter  :: num='-+0123456789.'
    character(len=10) :: firstchar
    integer :: ieq, ipm,iostat
    character(len=*), parameter :: esi  = 'Event-shape identifier = '
    character(len=*), parameter :: pdfi = 'PDF identifier = '

    firstchar = adjustl(line(1:10))
    if (index(num, firstchar(1:1)) > 0) then
       ltype = ltype_3num
       read(line,*) vals(1:3)
       return
    end if

    if (index(line, '==') > 0) then
       ltype = ltype_nop
       return
    end if

    !-- event-shape indentifier
    ieq = index(line,esi)
    if ( ieq > 1 ) then
       ltype = ltype_esname
       vals(1) = es_idOfName(trim(line(ieq + len(esi):)))
       return
    end if
    
    !-- pdf identifier
    ieq = index(line,pdfi)
    if ( ieq > 1 ) then
       ltype = ltype_pdfname
       vals(1) = pn_Name2ID(trim(line(ieq + len(pdfi):)))
       return
    end if

    ieq = index(line, '=')
    if (ieq > 0) then
       read(line(ieq+1:),*) vals(1)
       ipm = index(line, '+-')
       if (ipm > ieq) then
          ltype = ltype_valerr
          read(line(ipm+2:),*) vals(2)
       else
          ltype = ltype_val
       end if
       return
    end if
    
    !-- default --
    ltype = ltype_nop
  end subroutine SplitIt
  
  
  !----------------------------------------------------------------
  ! searches for the label until the end of the file 
  ! (using this presumes that the order of labels is well-defined
  function label_val(idev,line,label) result(res)
    integer,          intent(in)    :: idev
    character(len=*), intent(inout) :: line
    character(len=*), intent(in)    :: label
    real(dp)                     :: res
    integer :: iostat, ix
    
    do
       ix = index(line, label)
       if (ix > 1) then
          read(line(ix+len(label):), *) res
          exit
       end if

       read(idev, '(a)', iostat=iostat) line
       if (iostat /= 0) then
          write(0,*) 'ERROR in label_val: requested label "'//label//'" not&
               & present'
          stop
       end if
    end do
  end function label_val


  function label_val_PDF(idev,line,label) result(res)
    use pdf_names
    integer,          intent(in)    :: idev
    character(len=*), intent(inout) :: line
    character(len=*), intent(in)    :: label
    real(dp)                     :: res
    integer :: iostat, ix
    
    do
       ix = index(line, label)
       if (ix > 1) then
          res = pn_Name2ID(trim(adjustl(line(ix+len(label):))))
          exit
       end if

       read(idev, '(a)', iostat=iostat) line
       if (iostat /= 0) then
          write(0,*) 'ERROR in label_val: requested label "'//label//'" not&
               & present'
          stop
       end if
    end do
  end function label_val_PDF

  !---------------------------------------------------
  function label_val_line(line,label) result(res)
    character(len=*), intent(inout) :: line
    character(len=*), intent(in)    :: label
    real(dp)                     :: res
    integer :: iostat, ix
    
    ix = index(line, label)
    if (ix > 1) then
       read(line(ix+len(label):), *) res
    else
       res = -1e90_dp
    end if
  end function label_val_line
  

end module join_genhelper
