!======================================================================
! Resummations "made easy"!
program main_parse
  use types; use consts_dp; use qcd
  !-- for some obscure reason osf likes to have conv_objects
  !   come before the rest...
  use conv_objects
  use matching_cache
  use parser; use holders; use convolution; use parton_distributions
  use event_shapes; use pdf_tabulate; use fo_evshp
  use resummed_results
  use interpolation
  use sub_defs_io
  use alfas_server
  use resummation_manager; use resummation_definitions
  use brdzE
  implicit none
  !--------------------------------------------
  type(fixed_order_set)  :: set
  type(fixed_order_evshp), pointer :: evarr(:)
  type(fixed_order_evshp), pointer :: evshp 
  type(qtab_holder)                :: qtabh
  integer :: i, n
  real(dp) :: xmin
  type(match_cache) :: mcache
  type(as_handle)   :: ash
  integer               :: nv, nres, ires
  integer,  parameter   :: nres_max=6
  real(dp), allocatable :: res(:,:), var_vals(:)
  real(dp)           :: var, dvar, base, maxvar
  integer            :: ixQbin, evshp_id, idev, ixQlo, ixQhi
  character(len=20)  :: evshp_name
  real(dp), parameter :: Qbin_factor = 10.0_dp
  !-- these are the parameters which will be modified ---------
  integer  :: lcl_PowerType(nres_max), lcl_MatchType(nres_max)
  integer  :: lcl_ESSumType(nres_max), lcl_esC1MuF(nres_max)
  integer  :: lcl_EvlnType(nres_max)
  real(dp) :: muR_Q(nres_max), rescale(nres_max)
  real(dp) :: lcl_esPTModPower(nres_max), lcl_esNPModPower(nres_max)
  !-- some common ones?
  logical  :: binned
  real(dp) :: alfa0

  write(0,*) '---------- parsing ------------'
  idev = idev_open_opt('-FOFile','h1-fixeds.top')
  call parser_ReadFile(idev, set)
  close(idev)
  call qcd_SetNf(set%nf)
  call qcd_SetGroup(set%ca, set%cf, set%tr)
  !-----------------------------------------


  !-- establish details of the event shape to be dealt with ----
  evshp_name = string_val_opt('-evshp','brd_zE')
  write(0,*) 'Will attempt to generate distributions for ',trim(evshp_name)
  evshp_id = es_idOfName(trim(evshp_name))
  write(0,*) '----------------------------------------;'

  call foe_ListToArray(set%first, evarr, es_ids=(/ evshp_id /),&
       & Qmin=5.0_dp, Qmax=100.0_dp)
  n = size(evarr)
  xmin = minval(evarr%xbj)
  call mca_InitCache(mcache, xmin, dy=0.05_dp, factscheme=set%factscheme)
  call foe_SetupMH(mcache, evarr)


  !-- use this one for testing muF and muR dependences
  call as_init(ash,  dble_val_opt('-as',0.118_dp), fixnf=nf_int)
  alfa0 = dble_val_opt('-alfa0',0.5_dp)

  !-- (lowest Q bin - 1) for which to actually print something useful.
  ixQlo = int_val_opt('-ixQlo',0)
  ixQhi = int_val_opt('-ixQhi',100)

  !-- some defaults, which may be changed further down ------------
  rescale(:)          = one
  muR_Q(:)            = one
  lcl_esC1MuF(:)      = esC1MuF_VQ
  lcl_esPTModPower(:) = one
  lcl_esNPModPower(:) = two
  lcl_EvlnType(:)     = EvlnType_LO
  !lcl_ESSumType(:)    = ESSumType_standard
  binned              = .false.

  !-- set the max value of the variable on the basis of info that
  !   is already available. It can be reset later?
  maxvar = dble_val_opt('-maxvar',evarr(1)%mh%es_info%esmax)

  !-- set up the different run options ----------------------
  if (log_val_opt('-FO+resum')) then
     nres = 3; nv = 500
     lcl_PowerType = PowerType_none
     lcl_MatchType(1) = MatchType_NoResum
     lcl_MatchType(2) = MatchType_None
     lcl_MatchType(3) = MatchType_Mmod
  else if (log_val_opt('-3resum')) then
     ! analyticall extend lambda 
     call brdzE_SetExtendedLambda(.true.)
     nres = 3; nv = 500
     lcl_PowerType(:) = PowerType_none
     lcl_MatchType(:) = MatchType_Mmod
     lcl_ESSumType(1) = ESSumType_standard
     lcl_ESSumType(2) = ESSumType_WithLbar_m 
     lcl_ESSumType(3) = ESSumType_WithLbar 
  else if (log_val_opt('-3power')) then
     nres = 3; nv = 500
     lcl_MatchType(:) = MatchType_Mmod
     lcl_PowerType(1) = PowerType_none
     lcl_PowerType(2:3) = PowerType_ModShift
     lcl_esNPModPower(2) = one
     lcl_esNPModPower(3) = two
  else if (log_val_opt('-binned')) then
     nres = 1; nv = 10
     binned = .true.
     lcl_MatchType(:) = MatchType_Mmod
     lcl_PowerType(:) = PowerType_ModShift
  else if (log_val_opt('-unevenMRST')) then
     nres = 2; nv = 500
     lcl_PowerType = PowerType_none
     lcl_MatchType = MatchType_Mmod
     lcl_esC1MuF(:)  = esC1MuF_Q ! VQ not supported with MRST
     lcl_EvlnType(1) = EvlnType_NLO
     lcl_EvlnType(2) = EvlnType_fromPRM
  else if (log_val_opt('-muREffect')) then
     nres = 6; nv = 500
     lcl_MatchType = MatchType_Mmod
     lcl_PowerType(1:3) = PowerType_none
     lcl_PowerType(4:6) = PowerType_ModShift
     muR_Q(1) = half
     muR_Q(2) = one
     muR_Q(3) = two
     muR_Q(4) = half
     muR_Q(5) = one
     muR_Q(6) = two
  else
     write(0,*) 'ERROR: must specify a command for what to generate'
     stop
  end if

  if (nres > nres_max) write(0,*) 'ERROR: nres is too large'
  allocate(res(0:nv,nres), var_vals(0:nv))

  write(6,'(a)') '# Gnrtd by: &
       &$Id: main-broadpaper.f90,v 1.4 2001/09/30 18:57:53 salam Exp $'
  do ixQbin = 1, size(evarr)
     !-- if we do not want lo Q bins, then add fake info to simulate
     ! the non-existent bins
     if (ixQbin < ixQlo+1 .or. ixQbin > ixQhi+1) then
        res(0,:) = zero
        if (binned) then
           write(6,'(10es18.9)') zero, zero, res(0,:)
        else
           write(6,'(10es18.9)') zero, res(0,:)
        end if
        write(6,*)
        write(6,*)
        cycle
     end if
     

     evshp => evarr(ixQbin) !size(evarr)

     write(0,*) 'Doing bin',ixQbin-1,' at x =',evshp%xbj,', Q = ',evshp%Q
     dvar = maxvar/nv; base = 0.0
     var_vals = (/ (i*dvar + base, i = 0, nv) /)

     !-- now iterate over various options
     do ires = 1, nres
        call es_UpdateDynamic(esRescaleV, evshp_id, rescale(ires))
        call es_UpdateDynamic(esSumType,  evshp_id, lcl_ESSumType(ires))
        call es_UpdateDynamic(esPTModPower, evshp_id, lcl_esPTModPower(ires))
        call es_UpdateDynamic(esNPModPower, evshp_id, lcl_esNPModPower(ires))
        call es_UpdateDynamic(esC1MuF, evshp_id, lcl_esC1MuF(ires))

        call rm_FOEvshpResum(evshp,var_vals, res(:,ires), ash=ash, &
             &muR_Q     = muR_Q(ires),&
             &MatchType = lcl_MatchType(ires), &
             &EvlnType  = lcl_EvlnType(ires), &
             &PowerType = lcl_PowerType(ires), &
             &alfa0=alfa0)
     end do
     
     res = res  * (Qbin_factor)**(ixQbin-1)

     do i = 1, nv
        if (binned) then
           write(6,'(10es18.9)') var_vals(i-1), var_vals(i),&
                &     (res(i,:) - res(i-1,:))/dvar!, res(i,:)
        else
           write(6,'(10es18.9)') half*(var_vals(i-1)+var_vals(i)),&
                &     (res(i,:) - res(i-1,:))/dvar!, res(i,:)
        end if
     end do
     write(6,*) 
     write(6,*) 
  end do
  
  stop
  
end program main_parse
