module mf_helper
  use types; use consts_dp
  use driver_withdsm_hidden
  use resummation_definitions; use event_shapes
  use alfas_server
  use sub_defs_io
  use warnings_and_errors
  implicit none
  private

  integer :: FOE_handle_lo, FOE_handle_hi, evshp_id
  
  !-- keep the results here?
  !   (vlo,vhi,res,err,thr) * 1:nbin * 0:nQ
  real(dp), allocatable :: exp_res(:,:,:), xQvals(:,:), Qvals(:,:)
  logical,  allocatable :: mask(:,:)
  integer :: nbin, nQ
  !-- these are the resummation properties
  integer :: PowerType, MatchType, EvlnType
  real(dp) :: muR_Q, muI
  real(dp) :: Qmin, Qmax, VQmin, Vmax
  real(dp), public :: LastAlfas, LastAlfa0, LastChi2=0
  real(dp) :: LastAsErr = zero, LastA0Err = zero
  integer  :: LastNpnt=0

  real(dp), parameter :: alfas_default = 0.118_dp
  real(dp), parameter :: alfa0_default = 0.500_dp

  character(len=40) :: FOFile
  character(len=6) :: evshp_name

  public :: mfh_Start, mfh_GetThr, mfh_Write, mfh_WriteProp, mfh_DoFit
  public :: mfh_WriteCont, mfh_WriteHiRes
contains

  !-- takes care of initialisation ---------------
  subroutine mfh_Start
    integer  :: FOE_handle, i, iQ, ibin
    real(dp) :: xbj, ybj,Q
    evshp_name = string_val_opt('-evshp','tau_tE')
    write(0,*) 'Will attempt to do a fit for ',trim(evshp_name)
    evshp_id = es_idOfName(trim(evshp_name))
    
    !-- use interface for the lazy ---------
    FOFile = string_val_opt('-FOFile', 'h1-fixeds.top')
    call dwdh_start(FOFile, evshp_id, FOE_handle_lo, FOE_handle_hi)

    !-- get the data!
    call hzread(evshp_name)

    !-- set up once and for all other things that are needed throughout
    nbin = size(exp_res,dim=2)
    nQ   = ubound(exp_res,dim=3)
    allocate (xQvals(2, 0:nQ))
    do FOE_handle = FOE_handle_lo, FOE_handle_hi
       call dwdh_info(FOE_handle, xbj,ybj,Q,i)
       xQvals(:, FOE_handle - FOE_handle_lo) = (/ xbj, Q /)
       if (log_val_opt('-printQvals')) &
            &write(0,*) 'bin ', FOE_handle - FOE_handle_lo, ': Q = ',Q 
    end do

    !-- set up properties of resummation ------------------------------
    !MatchType = MatchType_NoResum
    MatchType = MatchType_logRmod
    !MatchType = MatchType_none
    !MatchType = MatchType_Mmod
    !MatchType = MatchType_M2mod
    MatchType = int_val_opt('-match', MatchType)
    PowerType = int_val_opt('-power', PowerType_modshift)
    !EvlnType  = EvlnType_fromPRM
    !EvlnType  = EvlnType_NLO
    EvlnType  = int_val_opt('-evln', EvlnType_LO)
    muR_Q     = dble_val_opt('-muR',one)
    muI       = dble_val_opt('-muI',2.0_dp)
    call es_UpdateDynamic(esMax, EvshpId_brd_zE, 0.5_dp)
    call es_UpdateDynamic(esMax, EvshpId_tau_tE, 0.45_dp)
    if (log_val_opt('-esmax')) then
       call es_UpdateDynamic(esMax, evshp_id, dble_val_opt('-esmax'))
    end if
    call es_UpdateDynamic(esSumType, EvshpId_brd_zE, esSumType_standard)
    !call es_UpdateDynamic(esSumType, EvshpId_brd_zE, ESSumType_WithLbar_m)
    !call es_UpdateDynamic(esPTModPower, evshp_id, 1.0_dp)
    !call es_UpdateDynamic(esNPModPower, evshp_id, 2.0_dp)
    call es_UpdateDynamic(esPTModPower, evshp_id, &
         &                      dble_val_opt('-ptpow',1.0_dp))
    call es_UpdateDynamic(esNPModPower, evshp_id, &
         &                      dble_val_opt('-nppow',2.0_dp))
    call es_UpdateDynamic(esC1MuF, evshp_id, esC1MuF_VQ)
    !call es_UpdateDynamic(esC1MuF, evshp_id, esC1MuF_Q)
    if (log_val_opt('-rescale')) call &
         &es_UpdateDynamic(esRescaleV, evshp_id, dble_val_opt('-rescale'))
    
    if (.not. log_val_opt('-NonGlobal', .true.)) then
       call es_UpdateDynamic(esSumType, evshp_id, esSumType_NoNonGlobal)
    end if
    

    !-- set up the mask ----
    allocate(mask(1:nbin,0:nQ),Qvals(1:nbin,0:nQ))
    Qvals = spread(xQvals(2,:) , dim=1, ncopies=nbin) 
    Qmax = 100.0_dp
    select case (evshp_id)
    case(EvshpId_tau_tE)
       Qmin  = 20.0_dp
       VQmin = 2.0_dp
       Vmax  = 0.31_dp
    case(EvshpId_rho_E)
       Qmin  = 20.0_dp
       VQmin = 1.5_dp
       Vmax  = 0.20_dp
    case(EvshpId_tau_zE)
       Qmin  = 20.0_dp
       VQmin = 4.0_dp
       Vmax  = 0.90_dp
       !-- this helps the fit no end...
       !call es_UpdateDynamic(esRescaleV, evshp_id, half)
    case(EvshpId_brd_zE)
       !write(0,*) 'HHHHHHH'
       Qmin  = 20.0_dp
       VQmin = 3.0_dp
       Vmax  = 0.46_dp
    case(EvshpId_Cpr_E)
       Qmin  = 20.0_dp
       !VQmin = 6.0_dp
       VQmin = 4.0_dp
       Vmax  = 0.70_dp
    case default
       mask = .true.
    end select

    !-- allow user reset
    Qmin = dble_val_opt('-Qmin',Qmin)
    Qmax = dble_val_opt('-Qmax',Qmax)
    VQmin = dble_val_opt('-VQmin',VQmin)
    Vmax = dble_val_opt('-Vmax',Vmax)

    mask = (Qvals > Qmin) .and. (Qvals < Qmax) .and.&
         & (half*sum(exp_res(1:2,:,:),dim=1) > VQmin/Qvals) .and.&
         & (half*sum(exp_res(1:2,:,:),dim=1) < Vmax) 

    ! -- alllows at most one bin to be removed
    if (log_val_opt('-RemoveBinNumb')) then
       ibin = int_val_opt('-RemoveBinNumb')
       iQ = ibin / nbin
       ibin = ibin + 1 - nbin * iQ
       mask(ibin, iQ) = .false.
    end if
    ! -- alllows at most one bin to be removed
    if (log_val_opt('-AddBinNumb')) then
       ibin = int_val_opt('-AddBinNumb')
       iQ = ibin / nbin
       ibin = ibin + 1 - nbin * iQ
       mask(ibin, iQ) = .true.
    end if
    
  

    if (log_val_opt('-RemoveLeftmostBin')) then
       do iQ = 0, nQ
          do ibin = 1, nbin
             if (mask(ibin, iQ)) then
                mask(ibin, iQ) = .false.
                exit
             end if
          end do
       end do
    end if
    

    deallocate(Qvals)
  end subroutine mfh_Start


  !----------------------------------------------------------
  ! The routine needed by Minuit; fval must contain chi^2
  subroutine mfh_MinFunc(npar,grad,fval,xval,iflag,futil)
    integer,  intent(in)    :: npar, iflag
    real(dp), intent(inout) :: grad(npar), fval
    real(dp), intent(in)    :: xval(npar)
    real(dp), external      :: futil ! will not be used
    !---------------------------------------------------
    real(dp) :: chi2(size(mask,dim=1),size(mask,dim=2))
    
    ! generate the distribution
    call mfh_GetThr(xval(1),xval(2))
    ! work out the chi^2
    where (mask)
       chi2 = ((exp_res(3,:,:) - exp_res(5,:,:))/exp_res(4,:,:))**2
    elsewhere
       chi2 = zero
    end where
    fval = sum(chi2)
    !write(0,*) xval(1:2), fval
  end subroutine mfh_MinFunc
  
  !------------------------------------------------------
  ! Run the fitting software?
  subroutine mfh_DoFit
    integer :: unit_scr = 10, ierflag
    integer :: i
    character(len=10) :: parname
    real(dp)          :: val(2), err(2), bnd1, bnd2
    real(dp)          :: chi2, chi2_dist, errdef
    integer           :: ni, nx, istat
    integer           :: count(size(mask,dim=1), size(mask,dim=2))
    call mninit(5,unit_scr,7)
    call mnparm(1, "alfasMZ", 0.118_dp, 0.002_dp, zero, zero, ierflag)
    !call mnparm(1, "alfasMZ", 0.118_dp, 0.002_dp, 0.05_dp, 0.15_dp, ierflag)
    call mnparm(2, "alfa0",   0.500_dp, 0.050_dp, zero, zero, ierflag)

    !-- fix alfas
    if (log_val_opt('-fixas')) then
       val(1) = dble_val_opt('-fixas')
       call mnparm(1, "alfasMZ", val(1), &
            &0.002_dp, zero, zero, ierflag) 
       call mncomd(mfh_MinFunc,'fix 1',ierflag,0)
       write(0,*) 'Fixing alfasMZ = ', val(1)
    end if
    
    if (log_val_opt('-fixa0')) then
       val(2) = dble_val_opt('-fixa0')
       call mnparm(2, "alfa0", val(2), 0.050_dp, zero, zero, ierflag) 
       call mncomd(mfh_MinFunc,'fix 2',ierflag,0)
       write(0,*) 'Fixing alfa0 = ', val(2)
    end if
    
    call mncomd(mfh_MinFunc, "minimize", ierflag, 0)
    do i = 1, 2
       call mnpout(i, parname, val(i), err(i), bnd1, bnd2, ierflag)
       write(0, '(a,f7.4," +-",f7.4)') parname, val(i), err(i)
    end do
    call mnstat(chi2, chi2_dist,errdef,ni,nx,istat)
    where (mask)
       count = 1
    elsewhere
       count = 0
    end where

    LastAsErr = err(1)
    LastA0Err = err(2)
    LastChi2 = chi2; LastNpnt = sum(count)
    write(0,'("chi2 = ",f8.2," for",i4," points")') chi2, LastNpnt
    !-- make sure we have the right distribution
    call mfh_GetThr(val(1),val(2))
  end subroutine mfh_DoFit
  
  !----------------------------------------------------------
  ! Writes all the details of the fit, including a leading #
  ! so that it can be part of a gnuplot file.
  subroutine mfh_WriteProp(idev)
    integer, intent(in) :: idev
    integer  :: ival
    real(dp) :: dval

    write(idev,'("# alfas(MZ): ",f8.6," +- ",f8.6)') LastAlfas, LastAsErr
    write(idev,'("# alfa0:     ",f8.6," +- ",f8.6)') LastAlfa0, LastA0Err
    write(idev,'("# chi2 = ",f8.2," for",i4," points")') LastChi2, LastNpnt

    write(idev,'("#")')
    write(idev,'("# muR_Q:     ",f8.6)') muR_Q
    write(idev,'("# muI:       ",f8.6)') muI

    write(idev,'("#")')
    write(idev,'("# FOFile:    ",a)') FOFile
    write(idev,'("# Evshp:     ",a)') evshp_name
    
    write(idev,'("#")')
    write(idev,'("# MatchType:     ",i2)') MatchType
    write(idev,'("# PowerType:     ",i2)') PowerType
    write(idev,'("# EvlnType:      ",i2)') EvlnType

    call es_GetDynamic(esSumType, evshp_id, ival)
    write(idev,'("# esSumType:      ",i2)') ival
    call es_GetDynamic(esC1MuF, evshp_id, ival)
    write(idev,'("# esC1MuF:        ",i2)') ival

    call es_GetDynamic(esRescaleV, evshp_id, dval)
    write(idev,'("# esRescaleV:     ",f9.6)') dval
    call es_GetDynamic(esMax, evshp_id, dval)
    write(idev,'("# esMax:          ",f9.6)') dval
    call es_GetDynamic(esPTModPower, evshp_id, dval)
    write(idev,'("# esPTModPower:   ",f9.6)') dval
    call es_GetDynamic(esNPModPower, evshp_id, dval)
    write(idev,'("# esNPModPower:   ",f9.6)') dval
    
    write(idev,'("#")')
    write(idev,'("# Qmin:      ",f10.6)') Qmin
    write(idev,'("# Qmax:      ",f10.6)') Qmax
    write(idev,'("# VQmin:     ",f10.6)') VQmin
    write(idev,'("# Vmax:      ",f10.6)') Vmax
    
    write(idev,'("#")')
    write(idev,'("# ",a)') trim(command_line())

  end subroutine mfh_WriteProp
  

  !---------------------------------------------------------------
  ! Though it does not strictly fit into the program, it is useful
  ! to have the option of outputting hi-res theory curves (e.g. 
  ! for investigating uncertainties, etc...)
  subroutine mfh_WriteHiRes(idev)
    integer, intent(in) :: idev
    !---------------------------
    type(as_handle)   :: ash
    integer, parameter :: nbin_hires = 200
    real(dp) :: var_vals(0:nbin_hires), sigma_vals(0:nbin_hires), alfas
    real(dp) :: Qval, evshp_max
    integer  :: iQ, FOE_handle, i

    call as_init(ash, LastAlfas)
    call es_GetDynamicParam(esMax, evshp_id, evshp_max)
    do i = 0, nbin_hires
       var_vals(i) = i * (evshp_max / nbin_hires)
    end do

    do FOE_handle = FOE_handle_lo, FOE_handle_hi
       iQ = FOE_handle - FOE_handle_lo
       Qval  = xQvals(2,iQ)
       if (Qval < Qmin .or. Qval > Qmax) cycle
       alfas = as_value(ash, Qval*muR_Q)
       call dwdh_dist(FOE_handle, var_vals, sigma_vals, alfas, muR_Q, &
            &LastAlfa0, muI, MatchType, EvlnType, PowerType)
       do i = 1, nbin_hires
          write(idev,'(2es14.6)') half*(var_vals(i-1)+var_vals(i)),&
               & (sigma_vals(i) - sigma_vals(i-1))&
               & / (var_vals(i) - var_vals(i-1))
       end do
    end do

  end subroutine mfh_WriteHiRes
  
  !-------------------------------------------------------------
  ! write out the results of the analysis
  subroutine mfh_Write(idev)
    integer, intent(in) :: idev
    integer :: iQ, i
    real(dp) :: Qfact = 10.0_dp
    do iQ = 0, nQ
       do i = 1, nbin
          !useit = .not.(Qval < 10.0_dp .or. exp_res(2,i,iQ) < 2.0_dp/Qval &
          !     & .or. exp_res(2,i,iQ) > 0.31_dp)
          !if (.not. useit) cycle
          write(idev,'(8es14.5,l,i2)') exp_res(1:2,i,iQ), &
               &exp_res(3:8,i,iQ)*Qfact**iQ, mask(i,iQ), iQ
          !write(idev,'(5es14.5,l)') exp_res(1:2,i,iQ), &
          !     &exp_res(3:5,i,iQ)*Qfact**(nQ-iQ), mask(i,iQ)
       end do
       write(idev,*)
       write(idev,*)
    end do
  end subroutine mfh_Write
  
  !-------------------------------------------------------------
  ! write out a 1-sigma error contour 
  subroutine mfh_WriteCont(idev)
    integer, intent(in) :: idev
    !------------------------------
    integer, parameter :: npnt = 100
    integer :: i, imod, ierflag, nfound
    real(dp) :: chi2, asmz(npnt), a0c(npnt), nstd(1)
    nstd = one
    call mnexcm(chi2,"set err",nstd,1,ierflag,0)
    call mncont(mfh_MinFunc, 1, 2, npnt, asmz, a0c, nfound, 0)
    !-- tricks
    do i = npnt/4, npnt+npnt/4
       imod = mod(i-1,npnt-1)+1
       write(idev,*) asmz(imod), a0c(imod)
    end do

  end subroutine mfh_WriteCont
  

  !-------------------------------------------------------------
  ! set up the theory prediction in entry 5 of the exp_res array
  subroutine mfh_GetThr(alfas_MZ, alfa0)
    real(dp), intent(in) :: alfas_MZ, alfa0
    !------------------------------------
    type(as_handle)   :: ash
    real(dp) :: var_vals(0:nbin), sigma_vals(0:nbin), alfas
    integer  :: iQ, FOE_handle, i
    real(dp) :: Qval

    LastAlfas = alfas_MZ
    LastAlfa0 = alfa0

    !-- these could have been set up once and for all?
    var_vals(0)  = exp_res(1,1,0)
    var_vals(1:) = exp_res(2,1:nbin,0)

    call as_init(ash, alfas_MZ)

    do FOE_handle = FOE_handle_lo, FOE_handle_hi
       iQ = FOE_handle - FOE_handle_lo
       Qval  = xQvals(2,iQ)
       !alfas = as_value(ash, Qval)
       alfas = as_value(ash, Qval*muR_Q)
       call dwdh_dist(FOE_handle, var_vals, sigma_vals, alfas, muR_Q, &
            &alfa0, muI, MatchType, EvlnType, PowerType)
!!$       call dwdh_dist_ash(FOE_handle, var_vals, sigma_vals, ash, muR_Q, &
!!$            &alfa0, muI, MatchType, EvlnType, PowerType)
       exp_res(5,:,iQ) = (sigma_vals(1:nbin) - sigma_vals(0:nbin-1))/&
            &(var_vals(1:nbin) - var_vals(0:nbin-1))
    end do
    
  end subroutine mfh_GetThr
  

  !------ read in the data ---------------------
  subroutine hzread(evshp_name)
    character(len=6), intent(in) :: evshp_name
    integer :: idev, i, j, found
    character(len=120) :: line=''
    real(dp) :: res(7)

    idev = get_new_device()

    if (trim(evshp_name) == 'rho_E') then
       open(unit=idev,file='h1-rho0.dat')
       !allocate(exp_res(1:5, 1:20, 0:7))
       ! col1 = Vlo, Vhi, ExpRes, ExpErr, Thr, ExpStat, 
       !                      Exp(stat+syst_+), Exp(stat+syst_-)
       allocate(exp_res(1:8, 1:20, 0:7))
       !-- initial stuff
       do i = 1, 3
          read(idev,*)
       end do
       !-- from here it cycles
       do j = 0, 7
          do i = 1, 4
             read(idev,*)
          end do
          do i = 1, 20
             read(idev,*) res(1), res(3), res(4:6)
             res(2) = half*(res(1) + res(3))
             res(7) = res(6)
             !-- add errors in quadrature...
             exp_res(:,i,j) = (/ res(1), res(3), res(4),&
                  & sqrt(res(5)**2 + (half*(abs(res(6))+abs(res(7))))**2), zero,&
                  & res(5), &
                  & sqrt(res(5)**2+res(6)**2), sqrt(res(5)**2+res(7)**2) /)
          end do
       end do
    else
       open(unit=idev,file='hz99193dist.dat')
       line = ''
       found = which_key(idev, line, trim(evshp_name))
       if (found == 0) call wae_error(&
            &'h1parse: could not find event shape '//trim(evshp_name))
       allocate(exp_res(1:8, 1:10, 0:7))
       !-- skip lines
       do j = 0, 7
          do i = 1, 6
             read(idev,'(a)') line
          end do
          do i = 1, 10
             read(idev,*) res
             res(4:7) = res(4:7)
             !-- add errors in quadrature...
             exp_res(:,i,j) = (/ res(1), res(3), res(4),&
                  & sqrt(res(5)**2 + (half*(abs(res(6))+abs(res(7))))**2), zero,&
                  & res(5), &
                  & sqrt(res(5)**2+res(6)**2), sqrt(res(5)**2+res(7)**2) /)
          end do
       end do
    end if
  end subroutine hzread
  
  function which_key(idev, line, key1) result(res)
    integer,          intent(in)    :: idev
    character(len=*), intent(inout) :: line
    character(len=*), intent(in)    :: key1
    integer :: res
    integer :: iostat, ix
    
    do 
       ix = index(line, key1)
       if (ix > 0) then
          res = 1; exit
       end if
       read(idev, '(a)', iostat=iostat) line
       if (iostat /= 0) then
          res = 0; exit
       end if
    end do
  end function which_key

end module mf_helper


program main_fitdist
  use mf_helper; use types; use sub_defs_io
  implicit none
  logical :: checkallargs, docontour 
  integer :: idev, idev_cont
  logical :: hires

  call mfh_start

  if (log_val_opt('-outbase')) then
     idev = idev_open_opt('-outbase',ext='.res')
  else
     idev = 6
  end if

  hires = log_val_opt('-hires')

  if (hires) then
     LastAlfas = dble_val_opt('-alfas',0.118_dp)
     LastAlfa0 = dble_val_opt('-alfa0',0.500_dp)
     call mfh_WriteProp(idev)
     call mfh_WriteHiRes(idev)
  else
     docontour = log_val_opt('-cont')
     if (log_val_opt('-fit',.true.)) then
        call mfh_DoFit
     else
        call mfh_GetThr(0.118_dp, 0.5_dp)
        write(0,*) 'Not doing a fit'
     end if
     
     call mfh_WriteProp(idev)
     call mfh_Write(idev)
     
     if (docontour) then
        idev_cont = idev_open_opt('-outbase',ext='.cnt')
        call mfh_WriteCont(idev_cont)
     end if
  end if
  

  checkallargs = CheckAllArgsUsed(0)

end program main_fitdist
