!======================================================================
! Test parsing
program main_parse
  use types; use consts_dp; use qcd
  use sub_defs_io
  !-- for some obscure reason osf likes to have conv_objects
  !   come before the rest...
  use conv_objects
  use matching_cache
  use parser; use holders; use convolution; use parton_distributions
  use event_shapes; use fo_evshp
  implicit none
  type(grid_def) :: gd
  type(fixed_order_set)  :: set
  type(fixed_order_evshp), pointer :: evarr(:)
  type(fixed_order_evshp), pointer :: evshp 
  type(sigma_holder)    :: sh
  type(gdval) :: xx
  integer :: n,i
  real(dp) :: xmin, rescale, Lmin
  real(dp), pointer :: pdf(:,:)
  type(match_cache) :: mcache
  logical :: lcl_itgtd


  write(0,*) '---------- parsing ------------'
  !-----------------------------------------
  call parser_ReadFile(5, set)
  write(0,*) 'nf is', set%nf
  call qcd_SetNf(set%nf)
  call qcd_SetGroup(set%ca, set%cf, set%tr)
  !-----------------------------------------
!!$  call foe_ListToArray(set%first, evarr, es_ids=(/EvshpId_tau_zQ,&
!!$       &EvshpId_tau_zE,&
!!$       & EvshpId_rho_E, EvshpId_Cpr_E, EvshpId_tau_tE,EvshpId_rho23E/))
!!$  call foe_ListToArray(set%first, evarr, es_ids=(/EvshpId_tau_zE,&
!!$       &EvshpId_brd_zE/))
  call foe_ListToArray(set%first, evarr, es_ids=(/ EvshpId_rho_E,&
       & EvshpId_Cpr_E, EvshpId_tau_tE, EvshpId_tau_zE,  EvshpId_brd_zE /))
!!$  call foe_ListToArray(set%first, evarr, es_ids=(/ EvshpId_tau_zQ,&
!!$       & EvshpId_rho_E, EvshpId_Cpr_E, EvshpId_tau_tE /))
  n = size(evarr)
  xmin = minval(evarr%xbj)
  if (log_val_opt('-smooth', .false.)) then
     write(0,*) 'WARNING: SMOOTHING BEGIN DONE'
     do i = 1, n
        call smooth_it(evarr(i))
     end do
  end if
  
  lcl_itgtd = log_val_opt('-itgtd')
  Lmin      = dble_val_opt('-Lmin', -1e4_dp)

  if (log_val_opt('-cfscale',.false.)) then
     rescale = 4.0_dp/3.0_dp / cf
     write(0,*) 'Will rescale by', rescale
  else
     rescale = one
  end if
  
  call mca_InitCache(mcache, xmin, dy=0.02_dp, factscheme=set%factscheme)
  !call mca_InitCache(mcache, xmin, dy=0.02_dp, factscheme=factscheme_DIS)
  call foe_SetupMH(mcache, evarr)
  
  do i = 1, n
     evshp => evarr(i)
     if (evshp%evshp_id == EvshpId_brd_zE) then
        Lmin = -7.0_dp
     else
        Lmin = dble_val_opt('-Lmin', -1e4_dp)
     end if
     call TDOut(evshp, lcl_itgtd, 6)
     !call TDOut(evshp, .true., 6)
!!$     write(6,*) '------------'
!!$     write(6,*) evshp%xbj, evshp%ybj, evshp%pdf_id
!!$     write(6,*) evshp%sigma(2),'+-',evshp%sigerr(2)
  end do
  

contains
  subroutine TDOut(evshp, itgtd, dev)
    use sub_defs_io
    type(fixed_order_evshp), intent(in)  :: evshp
    logical                , intent(in)  :: itgtd
    integer,                 intent(in)  :: dev
    !------------------------------
    integer  :: ibin, nbin, iord
    real(dp), target :: sigmaR_1, sigmaR_2
    real(dp)         :: res, err, L, dL, absL, L0, L1
    real(dp), pointer :: dist(:,:), sigmaR
    character(len=120) :: line
    character(len=2), parameter :: ordchar(2) = (/ 'LO', 'NL' /)
    logical :: outlog = .true., first = .true.
    integer :: nstep = 1
    

    if (first) then
       nstep = int_val_opt('-nstep',1)
       write(0,*) 'nstep is', nstep
       if (.not. outlog) write(0,*) 'Warning: plotting var * dsigma/dvar'
       first = .false.
    end if
    
    !write(0,*) 'GCA12 in TDOUT is', evshp%mh%GCA12, es_names(evshp%evshp_id)
    !write(0,*) 'CCA2L in TDOUT is', evshp%mh%CCA2L_1, es_names(evshp%evshp_id)

    do iord = 1, 2
       select case(iord)
       case(1)
          dist   => evshp%disti1
          sigmaR => sigmaR_1
       case(2)
          dist   => evshp%disti2
          sigmaR => sigmaR_2
       end select
       
       nbin = size(dist,dim=1)-1

       write(dev,*) 'NEW FRAME'
       write(dev,*) 'SET WINDOW X 4.9 9.5 Y 2 9'
       write(dev,*) 'SET FONT DUPLEX'
       line = "TITLE TOP '"//es_names(evshp%evshp_id)//' '&
            & //ordchar(iord)//' '//trim(num2char(evshp%pdf_id))//"'"
       !write(line,*) 
       write(dev,*) trim(line)
       if (outlog) then
          write(dev,*) 'SET LIMITS X',log(dist(0,1)), log(dist(nbin,1))
       else
          write(dev,*) 'SET LIMITS X',(dist(0,1)), (dist(nbin,1))
       end if
       
       write(dev,*) 'SET PATT 0.02 0.08'
       write(dev,*) 'SET ORDER X Y DY'

       do ibin = 0, nbin, nstep
          if (itgtd) then
             L = log(dist(ibin  ,1))
             call ma_GetSigmas(evshp%mh, -L, sigmaR_1, sigmaR_2)
             res = dist(ibin,2) - sigmaR
             err = dist(ibin,3)
             if (L >= Lmin) write(dev,*) L, res, err
          else
             if (ibin < nstep) cycle
             L0 = log(dist(ibin-nstep,1))
             L1 = log(dist(ibin  ,1))
             L  = half*(L1 + L0)
             dL =      (L1 - L0)
             if (L < Lmin) cycle
             res = dist(ibin,2) - dist(ibin-nstep,2)
             call ma_GetSigmas(evshp%mh, -L0, sigmaR_1, sigmaR_2)
             res = res + sigmaR
             call ma_GetSigmas(evshp%mh, -L1, sigmaR_1, sigmaR_2)
             res = res - sigmaR
             if (outlog) then
                res = res / dL
                err = sqrt((dist(ibin-nstep,3)**2 - dist(ibin,3)**2))/dL
                if (L >= Lmin) write(dev,*) L, res*rescale, err*rescale
             else
                res = res / dL
                err = sqrt((dist(ibin-nstep,3)**2 - dist(ibin,3)**2))/dL
                if (L >= Lmin) write(dev,*) exp(L), res*rescale, err*rescale
             end if
             
          end if
          
       end do

       write(dev,*) 'HIST ; PLOT'
    end do
    

  end subroutine TDOut
  
  
  !----------------------------------------------------------------------
  ! attempt to get rid of nasty pieces
  subroutine smooth_it(evs)
    type(fixed_order_evshp), intent(inout) :: evs
    integer :: nbin, i

    nbin  = size(evs%dist2, dim=1)

    do i = nbin-1, 1, -1
       if (evs%dist2(i+1,3) /= zero .and. evs%dist2(i,3) > 10.0_dp*evs&
            & %dist2(i+1,3)) then
          write(0,*) 'smoothed'
          evs%dist2(i,2:3) = evs%dist2(i+1,2:3)
       end if
    end do
    

  end subroutine smooth_it
  

end program main_parse
