!======================================================================
program process
  use types; use consts_dp; use qcd
  use event_shapes
  use parton_distributions
  use coefficient_functions; use convolution_communicator!
  use convolution; use conv_objects
  use holders
  use alfas_server
  use matching
  implicit none

  
  integer, parameter :: factscheme = factscheme_msbar
  !--- for holding the fixed order MC results --------------------
  integer, parameter :: nmax = 400, nord = 2
  real(dp) :: fixed_order_diff(3,1:nmax,nord)
  real(dp) :: fixed_order_itgd(3,0:nmax,nord)
  real(dp) :: fixed_order_dvar(nord)
  integer  :: fixed_order_nbin(nord), nbin1, nbin2
  !--- for holding matching & parton distribution stuff -----------
  type(grid_def)          :: gd
  type(sigma_holder)      :: sh
  type(eventshape_holder) :: esh
  type(match_holder)      :: mh
  real(dp), pointer :: pdf(:,:)
  real(dp) ::  sigmaR2_1, sigmaR2_2, sigmaRL_1, sigmaRL_2
  !--- for holding the total cross section -----
  integer, parameter  :: nstruct = 20
  real(dp) :: sigtot(0:1,0:nstruct), Lvals_struct(0:nstruct)
  real(dp) :: dL_struct
  !--- details of what to do with alpha_s -------------------------
  type(as_handle) :: ash, ash_1loop
  real(dp)        :: alfaQ
  !-- the particular distribution in which we are interested -------
  real(dp) :: ymax = 15, dy = 0.05
  real(dp) :: scale = 81.3_dp, xbj = 1.8e-1_dp
  !real(dp) :: scale = 15.0_dp, xbj = 1.4e-2_dp
  !-- 
  logical :: fixed_logrmc


  if (nf /= 5) write(0,*) 'WARNING, nf should probably be equal to 5?'

  write(0,*) 'VERY BASIC MATCHING: ONLY F2 PART + OTHER RESTRICTIONS'
  write(0,*) 'No care taken for upper edge effects'
  write(0,*) 'Normalisation not properly taken care of'

  !---------- start up things related to parton distributions -----
  call conv_InitGridDef(gd, dy, ymax)
  call holder_InitSigma(gd, sh, factscheme)
  call holder_InitEventShape(esh, sh, EvshpId_tau_zE)
  xbj = 2.000000000000000E-005
  scale = 20.0000000000000
  !call pdf_InitDist(gd, pdf, scale, pdf_MRS99+1)
  call pdf_InitDist(gd, pdf, scale, pdf_POWER10g)

  write(0,*) conv_EvalGridQuant(gd, esh%C2_1 .conv. pdf, -log(xbj)) /&
       & conv_EvalGridQuant(gd, esh%C2 .conv. pdf, -log(xbj))


  call ma_InitMatchHolder(mh, sh, esh, pdf, xbj, ybj=zero, muF_Q=one)

  !---------- set up alpha_s --------------------------------------
  call as_init(ash, 0.118_dp)    ! at M_Z
  alfaQ = as_value(ash,scale)
  !    the alfas to be used for the parton evolution
  call as_init(ash_1loop, alfaQ, scale, nloop = 1, fixnf = 5)
  
  !-- DO ANYTHING REQUIRING PDF BEFORE HERE

  !--- set up the cross sections -------
  call prepare_sigma

  !---------- get the MC fixed order stuff ready ------------------
  call fixed_order_read
  call write_it_all

!!$  !-- remember this bit -----------------------------------------
!!$  call ma_GetSigmas(mh, esh, L, sigmaR2_1, sigmaR2_2, sigmaRL_1,&
!!$             & sigmaRL_2)
!!$  !----------------------------------------------------------------


  ! TO DO:
  !        consider subtracting the matching piece straight away?
  !        think up a routine for obtaining parton distribution at
  !        different scales...

 contains

   !-- normally would not approve of contained routine
   !   assume that binning is uniform in some variable or other...
   subroutine fixed_order_read
     integer  :: i, iord
     integer  :: nbin
     real(dp) :: dvar

     do iord = 1, 2
        read(5,*) nbin
        fixed_order_nbin(iord) = nbin
        do i = 1, nbin
           read(5,*) fixed_order_diff(:,i,iord)
        end do
        dvar = fixed_order_diff(1,nbin,iord) - fixed_order_diff(1,nbin-1,iord)
        fixed_order_dvar(iord) = dvar
        if (dvar <= 0) then
           write(0,*) 'ERROR: Bins are expected to be increasing'
           stop
        end if
        fixed_order_itgd(:,nbin,iord) = &
             &(/ fixed_order_diff(1,nbin,iord)+half*dvar, zero, zero /)
        do i = nbin-1, 0, -1
           fixed_order_itgd(1,i,iord) = fixed_order_itgd(1,i+1,iord) - dvar
           fixed_order_itgd(2,i,iord) = fixed_order_itgd(2,i+1,iord) &
                &- dvar*fixed_order_diff(2,i+1,iord)
           fixed_order_itgd(3,i,iord) = sqrt(fixed_order_itgd(3,i+1,iord)**2 &
                &+ (dvar*fixed_order_diff(3,i+1,iord))**2)
        end do
        !-- fairly ugly test to decide if we are logarithmic or not
        fixed_logrmc = fixed_order_itgd(1,0,iord) < -0.5_dp
!!$        write(0,*) 'End of fixed-order spectrum lies at',&
!!$             & fixed_order_itgd(1,0,iord),fixed_order_itgd(:,nbin,iord)
     end do
     
     nbin1 = fixed_order_nbin(1)
     nbin2 = fixed_order_nbin(2)
   end subroutine fixed_order_read
   

   !-------------------------------------------------------------
   ! prepare the LO cross section at a variety of different scales
   subroutine prepare_sigma
     use evolution
     integer  :: i
     real(dp) :: last_Q2, Q2
     !-- go down to 1 GeV 
     dL_struct = log(scale**2) / nstruct
     
     sigtot(0,0) = conv_EvalGridQuant(gd,sh%C2   .conv. pdf,-log(xbj))
     sigtot(1,0) = conv_EvalGridQuant(gd,sh%C2_1 .conv. pdf,-log(xbj))
     Lvals_struct(0) = zero

     Q2 = scale**2
     do i = 1, nstruct
        last_Q2 = Q2
        Q2      = exp(-dL_struct)*last_Q2
        !call ev_evolveLO(sh%P, pdf, ash_1loop, sqrt(last_Q2), sqrt(Q2))
        call ev_evolveNLO(sh%P, sh%P1, pdf, ash, sqrt(last_Q2), sqrt(Q2))
        !sigtot(0,i) = conv_EvalGridQuant(gd,pdf(:,1),-log(xbj))
        sigtot(0,i) = conv_EvalGridQuant(gd,sh%C2   .conv. pdf,-log(xbj))
        sigtot(1,i) = conv_EvalGridQuant(gd,sh%C2_1 .conv. pdf, -log(xbj))
        Lvals_struct(i) = i*dL_struct
     end do
     
   end subroutine prepare_sigma
   

   !----------------------------------------------------------------------
   subroutine write_it_all
     real(dp) :: L, tau
     real(dp) :: res
     integer  :: i
     integer, parameter :: order = 4, npt=100
    
     
     
     do i = 0, npt
        L = i* 10.0_dp / npt

        res = (matched_result(L+0.01_dp) - matched_result(L-0.01_dp)) / 0.02_dp
        write(6,'(7es20.10)') L, res
     end do
     write(0,*) 'Done'
   end subroutine write_it_all
   

   !----------------------------------------------------------------------
   ! just returnthe matched result...
   function matched_result(L) result(res)
     use interpolation
     real(dp), intent(in) :: L
     real(dp)             :: res
     real(dp) :: var
     real(dp) :: sig0, dsig0
     real(dp) :: sigE_1, dsigE_1
     real(dp) :: sigE_2, dsigE_2
     real(dp) :: sigR_1, sigR_2, dummy1, dummy2
     real(dp) :: asbar
     integer, parameter :: order = 4, npt=20
     logical, save :: first = .true.

!!$     call interpolate(Lvals_sigtot, sigtot(0,:),L,sig0,dsig0,order)

     if (fixed_logrmc) then
        var = -L
     else
        var = exp(-L)
     end if
     !write(0,*) 'hello', L
     call interpolate(fixed_order_itgd(1,:nbin1,1),&
          & fixed_order_itgd(2,:nbin1,1), var, sigE_1, dsigE_1, order)
     call interpolate(fixed_order_itgd(1,:nbin2,2),&
          & fixed_order_itgd(2,:nbin2,2), var, sigE_2, dsigE_2, order)
     
     call ma_GetSigmas(mh, esh, L, sigR_1, sigR_2, dummy1, dummy2)

     !asbar = alfaQ / twopi
     asbar = one
     if (first) then
        write(0,*) 'WARNING -- have put asbar = 1'
        first = .false.
     end if

     !write(0,*) L,sigR_2 / L**4, mh%F2_1(2)
     res = zero
     res = res + asbar    * (sigE_1 - sigR_1)
     res = (sigR_1)
     !res = res + asbar**2 * (sigE_2 - sigR_2)
     !res = sigR_2
     !res = res - asbar**2 * (sigE_1 - sigR_1) * (esh%G12*L**2 + esh%G11*L)
   end function matched_result
   

end program process
