!======================================================================
! Test parsing
program main_parse
  use types; use consts_dp; use qcd
  !-- for some obscure reason osf likes to have conv_objects
  !   come before the rest...
  use conv_objects
  use matching_cache
  use parser; use holders; use convolution; use parton_distributions
  use event_shapes; use pdf_tabulate; use fo_evshp
  use resummed_results; use resummation_definitions
  use sub_defs_io; use alfas_server
  implicit none
  !--------------------------------------------
  type(fixed_order_set)  :: set
  type(fixed_order_evshp), pointer :: evarr(:)
  type(fixed_order_evshp), pointer :: evshp 
  type(qtab_holder)                :: qtabh
  integer :: n,i
  real(dp) :: xmin
  type(match_cache) :: mcache
  real(dp) :: alfas, var, sigma1, sigma2, logvar, as2pi
  real(dp) :: sigmaR_1, sigmaR_2, rsmd_R, muFval
  real(dp) :: muR_Q, rescale


  write(0,*) '---------- parsing ------------'
  !-----------------------------------------
  call parser_ReadFile(5, set)
  write(0,*) 'nf is', set%nf
  call qcd_SetNf(set%nf)
  call qcd_SetGroup(set%ca, set%cf, set%tr)
  !-- changing beta0 is dangerous when the resummation involves
  !   evaluation of alphas at a different scale 
  !   (e.g through structure function evolution).
  !call qcd_SetGroup(set%ca, 0.3_dp, set%tr)
  !call qcd_SetGroup(set%ca, set%cf, 1e-3_dp)
  !-----------------------------------------
  !call foe_ListToArray(set%first, evarr, es_ids=(/ EvshpId_tau_zE /),&
!!$  call foe_ListToArray(set%first, evarr, es_ids=(/ EvshpId_tau_zE /),&
!!$       & Qmin=50.0_dp, Qmax=70.0_dp)
  call foe_ListToArray(set%first, evarr, es_ids=(/ EvshpId_brd_zE /),&
       & Qmin=50.0_dp, Qmax=70.0_dp)
!!$  call foe_ListToArray(set%first, evarr, es_ids=(/ EvshpId_brd_zE /),&
!!$       & Qmin=10.0_dp, Qmax=15.0_dp)

  n = size(evarr)
  xmin = minval(evarr%xbj)
  
!!$  !--- PLAY SOME VERY NAUGHTY GAMES HERE!!!
!!$  evarr(1)%evshp_id = EvshpId_tau_zQ

  call mca_InitCache(mcache, xmin, dy=0.01_dp, factscheme=set%factscheme)
  call foe_SetupMH(mcache, evarr)
  
  evshp => evarr(1)

  
  n = 100

  !-- some playing around to select a nice value of alphas
  !    using as(MZ) = 0.118, we have
  !--- Q = 8.1611e17  -> as(Q) = 0.02000
  !--- Q = 2.1155e37  -> as(Q) = 0.01000
  !--- Q = 1.6975e76  -> as(Q) = 0.00500
  alfas  = dble_val_opt('-as',0.2_dp)
  !call as_init(alfas)
  

  muR_Q = dble_val_opt('-muR' , one)
  rescale = dble_val_opt('-rescale',one)
  write(0,*) 'RESCALE IS', rescale
  !-- now fake things a little bit?
  muFval = evshp%mh%Q * evshp%mh%muF_Q
  if (evshp%mh%es_info%GP /= zero) then
     call qtab_InitQTAB(qtabh, evshp%mh%xbj, muFval*exp(-two)*rescale, muFval, &
          &qtabid_evlnNLO, mcache%sh, evshp%pdf, alfas, evshp%mh%Q*muR_Q,&
          &muR_Q, evshp%esh, evshp%ybj)
!!$     call qtab_InitQTAB(qtabh, evshp%mh%xbj, muFval*exp(-two), muFval, &
!!$          &qtabid_evlnNLO, mcache%sh, evshp%pdf, alfas, evshp%mh%Q*muR_Q)
  end if

  !-- change type of broadening resummation
  call es_UpdateDynamic(esSumType, evshp%evshp_id, esSumType_standard)
  !call es_UpdateDynamic(esSumType, EvshpId_brd_zE, esSumType_WithLbar)
  !-- modify behaviour relating to scale of structure function
  call es_UpdateDynamic(esC1muF, evshp%evshp_id, esC1muF_VQ)
  !-- rescale things
  call es_UpdateDynamic(esRescaleV, evshp%evshp_id, rescale)

  do i = 1, n
     logvar = i*2.0_dp/n !+ log(rescale)
     !write(0,*) i, logvar
     as2pi  = alfas/twopi
     rsmd_R = rr_R(evshp%mh, qtabh, exp(-logvar), alfas, &
          &zero,zero,muR_Q = muR_Q, MatchType=MatchType_none)
     call ma_GetSigmas(evshp%mh, logvar, sigmaR_1, sigmaR_2, &
          &muR_Q=muR_Q, nlo_evln=.true.) 
!!$     call ma_GetSigmas(evshp%mh, logvar, sigmaR_1, sigmaR_2, &
!!$          &muR_Q=muR_Q, nlo_evln=.true.) 
     write(6,'(4es25.15)') logvar, rsmd_R, &
          & (one + as2pi*(sigmaR_1 + as2pi*sigmaR_2))

     !.....

     ! alfas = i * 1e-4_dp / n
     ! var = 0.1_dp
     ! sigma1 = rsm_sigma(evshp%mh, alfas, var)
     ! var = 0.01_dp
     ! sigma2 = rsm_sigma(evshp%mh, alfas, var)
     ! write(6,'(4es20.12)') alfas, sigma1, sigma2
  end do



end program main_parse
