! $Id: matching.f90,v 1.19 2002/05/23 14:40:07 salam Exp $
module matching
  use types; use consts_dp
  !-- would have liked to have placed these in ma_InitMatchHolder, but
  !   the osfmite compiler gave a segmentation violation
  use convolution; use conv_objects
  use holders; use event_shapes
  use qcd
  implicit none
  private
  
  !----------------------------------------------------
  ! a type which holds the results of the convolutions
  type match_holder
     !-- to make it easier to write the final formulae we will actuall
     !   express a lot of these things as factors times q, even though
     !   they are actually convolutions...
     !
     !   Also hold F2L_1, which is not needed here (and more efficiently 
     !   calculated elsewhere) -- but this is a convenient place to keep it
     real(dp) :: F2_LO, F2L_1
     real(dp) :: P, PxP, P1, C2L_1, CCA2L_1, C2L1xP
     !-- it may be better (faster) to use the following?
     !   but also less flexible for future matching
     real(dp) :: F_1(0:2)
     real(dp) :: F_2(0:4)
     !-- how things will be normalised
     real(dp) :: norm
     !-- some labelling for easier tracking?
     real(dp) :: xbj, ybj, Q, muF_Q
     !-- copy these from esh ---------------
     type(evshp_info) :: es_info
     real(dp)         :: rescale, log_rescale
     integer          :: id
  end type match_holder
  
  public :: match_holder, ma_InitMatchHolder, ma_GetSigmas
contains
  

  !----------------------------------------------------------------------
  ! For a given parton distribution sets up the pieces needed for generic
  ! matching at the given x value, and any L or as value.
  !
  ! Formally not necessary to specify 0 lower bounds for pdf and P_x_pdf
  ! but it seems to be a workaround around a pgf90 (3.2-3) bug
  !
  !subroutine ma_InitMatchHolder(mh, sh, esh, pdf, x)
  subroutine ma_InitMatchHolder(mh, sh, esh, pdf, xbj, ybj, muF_Q, Q)
    use warnings_and_errors
    type(match_holder),      intent(inout), target :: mh
    type(sigma_holder),      intent(in)    :: sh
    type(EventShape_holder), intent(in)    :: esh
    real(dp),                intent(in)    :: pdf(0:,0:)
    real(dp),                intent(in)    :: xbj, ybj, muF_Q
    real(dp), optional,      intent(in)    :: Q
    !---------------------------------------------------
    real(dp) :: P_x_pdf(0:ubound(pdf,dim=1),0:ubound(pdf,dim=2))
    real(dp) :: y, C2_x_P_x_pdf
    integer, save      :: nwarn = 0
    integer, parameter :: nwarn_limit = 10
    !-- just a shortcut ---
    type(evshp_info), pointer :: ei
    

    !-- copy some stuff from esh    -----
    mh%es_info = esh%es_info
    ei         => mh%es_info
    !-- useful to have some redundant copies
    mh%id      = esh%id
    mh%rescale = mh%es_info%rescale
    if (ei%rescale <= zero) stop 'ERROR: rescale was <= zero'
    mh%log_rescale = log(ei%rescale)
    !-- some general reference info -----
    mh%xbj = xbj
    mh%ybj = ybj
    if (present(Q)) then
       mh%Q = Q
    else
       mh%Q = -one ! signal that it is unknown
    end if
    !-- have it in from a structural point of view, but do not use it yet.
    mh%muF_Q = muF_Q
!!$    if (mh%muF_Q /= one .and. ei%GP /= zero) call &
!!$       & wae_error('ma_InitMatchHolder:', &
!!$       &'muF_Q /= 1 is not yet supported for vars wrt z axis')

    y = -log(xbj)

    !-- need these if muF/=Q or if we have evolution
    if (muF_Q /= one .or. ei%GP /= zero) then
       P_x_pdf = sh%P .conv. pdf
       C2_x_P_x_pdf = conv_EvalGridQuant(sh%gd, esh%C2 .conv. P_x_pdf , y)
    else
       !-- avoid undefined values getting in the way, though 
       !   it should not actually matter(?)
       P_x_pdf      = zero
       C2_x_P_x_pdf = zero
    end if
    
    !-- set up the various elements which go into the matching formulas
    mh%F2_LO = conv_EvalGridQuant(sh%gd, esh%C2 .conv. pdf, y)
    mh%F2L_1 = cobj_Eval2LConv(sh%C2_1, sh%CL_1, pdf, xbj, ybj)
    mh%F2L_1 = mh%F2L_1 - two*log(muF_Q) * C2_x_P_x_pdf

    !-- changes of scale of quark distribution 
    if (ei%GP /= zero) then
       !P_x_pdf = sh%P .conv. pdf
       !mh%P   = ei%GP * conv_EvalGridQuant(sh%gd, esh%C2 .conv. P_x_pdf , y)
       mh%P   = ei%GP * C2_x_P_x_pdf
       mh%PxP = ei%GP**2 * conv_EvalGridQuant(sh%gd, esh%C2 .conv. &
            &      (sh%P .conv. P_x_pdf), y)
       mh%P1  = ei%GP * conv_EvalGridQuant(sh%gd, esh%C2 .conv. &
            &      (sh%P1 .conv. pdf) , y) 
       mh%C2L1xP = ei%GP * cobj_Eval2LConv(esh%C2_1, esh%CL_1, &
            &P_x_pdf, xbj, ybj) - two*log(muF_Q)*mh%PxP/ei%GP
    else
       mh%P   = zero
       mh%PxP = zero
       mh%P1  = zero
       mh%C2L1xP = zero
    end if

    mh%C2L_1  = cobj_Eval2LConv(esh%C2_1, esh%CL_1, pdf, xbj, ybj)
    mh%C2L_1 = mh%C2L_1 - two*log(muF_Q) * C2_x_P_x_pdf
    if (ei%resumCA) then
       mh%CCA2L_1 = cobj_Eval2LConv(esh%CCA2_1, esh%CCAL_1, pdf, xbj, ybj)
    else
       mh%CCA2L_1 = zero
    end if
    !write(0,*) "WATCH OUT: ALL Cs are being set to zero"
    !mh%C2L_1 = mh%F2L_1; mh%CCA2L_1 = zero

    !-- this allows one to cheat a little and pretend that convolutions
    !   are just like products
    if (mh%F2_LO == zero) then
       write(0,*) 'Error in ma_InitMatchHolder:&
            & zero quark distribution not supported' 
       stop
    end if    
    mh%P       = mh%P       / mh%F2_LO
    mh%PxP     = mh%PxP     / mh%F2_LO
    mh%P1      = mh%P1      / mh%F2_LO
    !-- give meaning of Mrinals C_q^q to C2L, at least for now...
    mh%C2L_1   = (mh%C2L_1 - mh%CCA2L_1) / mh%F2_LO
    mh%CCA2L_1 = mh%CCA2L_1 / mh%F2_LO
    mh%C2L1xP  = mh%C2L1xP  / mh%F2_LO

    !--- for cheating purposes ---------------------
    if (mh%F2_LO <= 1e-20) then
       if (nwarn <= nwarn_limit) then
          write(0,*) 'WARNING in ma_InitMatchHolder: &
               &results will be multiplied by 1e-40'
          if (nwarn == nwarn_limit) write(0,*)&
               &      '           further such messages will be suppressed' 
          nwarn = nwarn + 1
       end if
       mh%norm = 1e-40_dp
    else 
       mh%norm = one
    end if
    mh%F2L_1 = mh%norm * mh%F2L_1 / mh%F2_LO

    !-- set up the coefficients of the logs: faster end use?
    !   will need to think (?) about how to rewrite this if
    !   eventually want something spread over all x
    ! 
    !   For log(muF_Q)*mh%P dependence, see CCN19-71
    mh%F_1(0:2) = (/ mh%C2L_1+mh%CCA2L_1, ei%G11 - mh%P, ei%G12 /)
    mh%F_2(0:4) = (/ zero, &                                      ! 0
         &     mh%C2L_1 * ei%G11,&                                ! 1
         &         (half*mh%PxP - (ei%G11+pi*beta0*ei%GP)*mh%P)&  ! 2
         &           +(half*ei%G11**2+ei%G22) + mh%C2L_1*ei%G12&  ! 2 cont
         &           +mh%CCA2L_1 * ei%GCA12,&                     ! 2 cont
         &         -ei%G12 * mh%P + (ei%G23+ei%G11*ei%G12),&      ! 3
         &         half*(ei%G12)**2 /)                            ! 4
    
  end subroutine ma_InitMatchHolder


  !----------------------------------------------------------------------
  ! sigmaR_1 and sigmaR_2 are to be understood to be multipliable by 
  ! (as/2pi)^n
  !
  ! expnt1 is the coefficient of as/2pi in L g1(as L) + g2(as L)
  !
  ! sigmaRg_1 and sigmaRg_2 are the O(as) and O(as^2) pieces associated
  ! with a leading gluon in the current hemisphere
  subroutine ma_GetSigmas(mh, L_in, sigmaR_1, sigmaR_2, expnt1, &
       & sigmaRg_1, sigmaRg_2, nlo_evln, muR_Q)
    use warnings_and_errors
    type(match_holder), intent(in), target  :: mh
    real(dp),           intent(in)  :: L_in
    real(dp),           intent(out) :: sigmaR_1, sigmaR_2
    real(dp), optional, intent(out) :: expnt1
    real(dp), optional, intent(out) :: sigmaRg_1, sigmaRg_2
    logical,  optional, intent(in)  :: nlo_evln
    real(dp), optional, intent(in)  :: muR_Q
    
    !--------------------------------------------------
    real(dp) :: L, L2, L3, L4, LL(0:4)
    real(dp) :: fourpibeta_lnmuR_Q
    !logical,  parameter :: ready_made = .false.
    logical,  parameter :: ready_made = .true.
    logical, save      :: first
    integer, save      :: warn_id_Llt0 = 0, warn_id_ready = 0
    integer, parameter :: nwarn_limit = 4
    type(evshp_info), pointer   :: ei
    real(dp), parameter :: eps=1e-12_dp
    !-- for rescaled variables...
    real(dp) :: lnX, DeltaC1, DeltaG11, DeltaG22, DeltaC1_noP

    !-- handy shortcut
    ei => mh%es_info
       
    !-- rescale it...
    lnX = log(ei%esRescaleV)
    L = L_in + mh%log_rescale - lnX

    if (L < -eps) then
       call wae_warn(nwarn_limit, warn_id_Llt0,&
            & 'ma_GetSigmas: L is less than zero: is that safe?')
       !write(0,*) L_in, L
    end if
    !-- avoid calling wae_warn if not necessary
    if (.not. ready_made .and. first) then
       call wae_warn(1, warn_id_ready,&
            &'ma-GetSigmas: Not using ready-made')
       !write(0,*) mh%GCA12 / mh%G12
       !write(0,*) mh%F_2(2)
       !write(0,*) mh%G22
       first = .false.
    end if


    L2 = L * L
    L3 = L * L2
    L4 = L * L3
    LL = (/ one, L, L2, L3, L4 /)

    if (ready_made) then
       sigmaR_1 = sum(mh%F_1 * LL(0:2))
       sigmaR_2 = sum(mh%F_2 * LL(0:4))
    else
    !-- perhaps later on consider storing just the coefficients
    !   of the different powers of L...
       sigmaR_1 = (ei%G12 * L2 + ei%G11*L) - mh%P * L + mh%C2L_1+mh%CCA2L_1
       sigmaR_2 = (half*(ei%G12 * L2)**2 + (ei%G23+ei%G11*ei%G12)*L3&
            &     +(half*ei%G11**2+ei%G22)*L2) &
            &     +(-ei%G12 * mh%P * L3 + (half*mh%PxP - ei%G11*mh%P -pi&
            &         *beta0*ei%GP *mh%P)*L2)& 
            &     +mh%C2L_1 * (ei%G12*L2 + ei%G11*L) + mh%CCA2L_1*ei%GCA12*L2
    end if

    !-- sometimes we want to be able to support a non-standard 
    !   kind of resummation.
    select case (ei%esSumType)
    case(ESSumType_standard)
    case(ESSumType_NoNonGlobal)
       !-- remove the piece which accounts for non-global emissions
       sigmaR_2 = sigmaR_2 - ei%G22_NG*L2
    case(ESSumType_WithLbar,ESSumType_WithLbar_m)
       if (lnX /= zero) call wae_error('ma_GetSigmas: &
            &WithLbar not supported with esRescaleV /= 1')
       sigmaR_2 = sigmaR_2 + ei%ns20(ei%esSumType) &
            &  + ei%ns21(ei%esSumType) * L
       sigmaR_2 = sigmaR_2 + ei%ns20_GP(ei%esSumType)/ei%GP * mh%P&
            &+ ei%ns21_GP(ei%esSumType)/ei%GP * L * mh%P
    case default
       call wae_Error('ma_GetSigmas: unrecognized ei%esSumType')
    end select

    if (lnX /= zero) then
       DeltaC1_noP = ei%G12 * lnX**2 + (ei%G11)*lnX
       DeltaC1  = DeltaC1_noP - mh%P*lnX
       DeltaG11 = two * ei%G12 * lnX
       DeltaG22 = three * ei%G23 * lnX
       sigmaR_1 = sigmaR_1 + DeltaG11*L + DeltaC1
       sigmaR_2 = sigmaR_2 + DeltaG11*ei%G12 * L3 &
            &  + (DeltaG22+DeltaG11*ei%G11 + half*DeltaG11**2&
            &         -DeltaG11*mh%P+DeltaC1*ei%G12) * L2 &
            &  + (DeltaC1*ei%G11+DeltaG11*mh%C2L_1&
            &          + DeltaC1*DeltaG11) * L
    else
       !-- these are needed outside of here...
       DeltaC1_noP = zero
       DeltaC1  = zero
       DeltaG11 = zero
    end if
    !write(0,*) '------ DeltaC1', DeltaC1

    !-- allow for possibility of C1 multiplying q(V Q) rather q(Q)
    select case (ei%esC1MuF)
    case(esC1MuF_Q)
       ! do nothing
    case(esC1MuF_VQ)
       sigmaR_2 = sigmaR_2 - (mh%C2L1xP + DeltaC1_noP*mh%P - mh%PxP*lnX) * L
    case default
       call wae_Error('ma_GetSigmas: unrecognized ei%esC1MuF')
    end select
    

    
    !-- support variable renormalisation scale
    if (present(muR_Q)) then
       fourpibeta_lnmuR_Q = (two*twopi_beta0*log(muR_Q))
       sigmaR_2 = sigmaR_2 + ei%G12 * L2 * fourpibeta_lnmuR_Q
    else
       fourpibeta_lnmuR_Q = zero
    end if

    !-- separate this out from ready made because one does not know
    !   a priori if one is going to want it
    if (present(nlo_evln)) then
       if (nlo_evln) then
          sigmaR_2 = sigmaR_2 - (mh%P1 + fourpibeta_lnmuR_Q*mh%P) * L
          !-- for reason why this appears only with nlo_evln, see comment 
          !   dated 2001-05-01 in pdf_tabulate
          sigmaR_2 = sigmaR_2 + two*twopi_beta0*log(mh%muF_Q)*mh%P * L
       end if
    end if

    if (present(expnt1)) then
       expnt1 = (ei%G12 * L2 + (ei%G11+DeltaG11)*L) 
       !-- define exponent as containing the quark evolution as well
       !   bear in mind that exponentiation of this stuff will be valid
       !   only at first order
       if (ei%esC1MuF /= esC1MuF_Q) expnt1 = expnt1 - mh%P * L
       expnt1 = expnt1 * mh%norm
    end if

    if (present(sigmaRg_1)) sigmaRg_1 = mh%CCA2L_1 * mh%norm
    if (present(sigmaRg_2)) sigmaRg_2 = mh%CCA2L_1 * ei%GCA12*L2 * mh%norm
    
    !-- fixes things up for (almost) pure gluon
    sigmaR_1 = sigmaR_1 * mh%norm
    sigmaR_2 = sigmaR_2 * mh%norm
    
  end subroutine ma_GetSigmas

end module matching



!O !======================================================================
!O ! This module will be responsible for calculating, in as efficient
!O ! a way as possible the things related to matching.
!O 
!O 
!O !
!O !
!O !
!O !  Know q dist (and x value?)
!O !  Want many evaluations of matching piece with varying L and as
!O !
!O !  Will also need easy access to evolved quark distribution
!O !  (but leave that until later)
!O !
!O !
!O !  For now (14/06/2000) write a version which does only a single x value
!O 
!O !----------------------------------------------------------------------
!O module matching_old
!O   use types; use consts_dp
!O   !-- would have liked to have placed these in ma_InitMatchHolder, but
!O   !   the osfmite compiler gave a segmentation violation
!O   use convolution; use conv_objects
!O   use holders
!O   use qcd
!O   implicit none
!O   private
!O   
!O   !----------------------------------------------------
!O   ! a type which holds the results of the convolutions
!O   type match_holder
!O      !-- to make it easier to write the final formulae we will actuall
!O      !   express a lot of these things as factors times q, even though
!O      ! theyare actually convolutions...
!O      real(dp) :: qdist
!O      real(dp) :: P, PxP, C2_1, CL_1
!O      !-- it may be better (faster) to use the following?
!O      real(dp) :: F2_1(0:2)
!O      real(dp) :: F2_2(0:4)
!O      real(dp) :: FL_1(0:0)
!O      real(dp) :: FL_2(0:2)
!O      !-- how things will be normalised
!O      real(dp) :: norm
!O      !-- some labelling for easier tracking?
!O      real(dp) :: x
!O      !-- ids for the type of event shape
!O      integer  :: evs_id
!O   end type match_holder
!O   
!O   public :: match_holder, ma_InitMatchHolder, ma_GetSigmas
!O 
!O contains
!O 
!O   !----------------------------------------------------------------------
!O   ! For a given parton distribution sets up the pieces needed for generic
!O   ! matching at the given x value, and any L or as value.
!O   !subroutine ma_InitMatchHolder(mh, sh, esh, qdist, x)
!O   subroutine ma_InitMatchHolder(mh,sh, esh, qdist, x)
!O     type(match_holder),      intent(inout) :: mh
!O     type(sigma_holder),      intent(in)    :: sh
!O     type(EventShape_holder), intent(in)    :: esh
!O     real(dp),                intent(in)    :: qdist(:,:)
!O     real(dp),                intent(in)    :: x
!O     !---------------------------------------------------
!O     real(dp) :: P_x_qdist(size(qdist,dim=1),size(qdist,dim=2))
!O     real(dp) :: y
!O     integer, save      :: nwarn = 0
!O     integer, parameter :: nwarn_limit = 10
!O 
!O     if (esh%resumCA) then
!O        write(0,*) 'ERROR: matching module does yet support variables which&
!O             & resum CA part'
!O        stop
!O     end if
!O 
!O     y = -log(x)
!O     !-- set up the various elements which go into the matching formulas
!O     mh%qdist = conv_EvalGridQuant(sh%gd, esh%C2 .conv. qdist, y)
!O 
!O     P_x_qdist = sh%P .conv. qdist
!O     mh%P     = conv_EvalGridQuant(sh%gd, esh%C2 .conv. P_x_qdist, y)
!O     mh%PxP   = conv_EvalGridQuant(sh%gd, esh%C2 .conv. (sh%P .conv.&
!O          & P_x_qdist), y)
!O 
!O     mh%C2_1  = conv_EvalGridQuant(sh%gd, esh%C2_1 .conv. qdist, y)
!O     mh%CL_1  = conv_EvalGridQuant(sh%gd, esh%CL_1 .conv. qdist, y)
!O 
!O     !-- this allows one to cheat a little and pretend that convolutions
!O     !   are just like products
!O     mh%norm = mh%qdist
!O     if (mh%qdist == zero) then
!O        write(0,*) 'Error in ma_InitMatchHolder:&
!O             & zero quark distribution not supported' 
!O        stop
!O     end if
!O 
!O     if (mh%qdist <= 1e-20) then
!O        if (nwarn <= nwarn_limit) then
!O           write(0,*) 'WARNING in ma_InitMatchHolder: &
!O                &results will be multiplied by 1e-40'
!O           if (nwarn == nwarn_limit) write(0,*)&
!O                &      '           further such messages will be suppressed' 
!O           nwarn = nwarn + 1
!O        end if
!O        mh%norm = 1e-40_dp
!O     else 
!O        mh%norm = one
!O     end if
!O     
!O     mh%P    = mh%P     / mh%qdist
!O     mh%PxP  = mh%PxP   / mh%qdist
!O     mh%C2_1 = mh%C2_1  / mh%qdist
!O     mh%CL_1 = mh%CL_1  / mh%qdist
!O     
!O     !-- set up the coefficients of the logs: faster end use?
!O     !   will need to think (?) about how to rewrite this if
!O     !   eventually want something spread over all x
!O     mh%F2_1(0:2) = (/ mh%C2_1, esh%G11 - mh%P, esh%G12 /)
!O     mh%F2_2(0:4) = (/ zero, mh%C2_1 * esh%G11,&                     ! 0, 1
!O          &         (half*mh%PxP - (esh%G11+pi*beta0)*mh%P)&         ! 2
!O          &           +(half*esh%G11**2+esh%G22) + mh%C2_1*esh%G12,& ! 2 cont
!O          &         -esh%G12 * mh%P + (esh%G23+esh%G11*esh%G12),&    ! 3
!O          &         half*(esh%G12)**2 /)                             ! 4
!O     mh%FL_1(0:0) = (/ mh%CL_1 /) 
!O     mh%FL_2(0:2) = (/ zero,&                                   ! 0
!O          &         mh%CL_1 * esh%G11,&                         ! 1
!O          &         mh%CL_1 * esh%G12 /)                        ! 2
!O 
!O   end subroutine ma_InitMatchHolder
!O 
!O 
!O   !----------------------------------------------------------------------
!O   ! sigmaR1 and sigmaR2 are to be understood to be multipliable by 
!O   ! (as/2pi)^n
!O   subroutine ma_GetSigmas(mh, esh, L, sigmaR2_1, sigmaR2_2, sigmaRL_1,&
!O        & sigmaRL_2) 
!O     type(match_holder), intent(in)  :: mh
!O     type(eventshape_holder), intent(in) :: esh
!O     real(dp),           intent(in)  :: L
!O     real(dp),           intent(out) :: sigmaR2_1, sigmaR2_2
!O     real(dp),           intent(out) :: sigmaRL_1, sigmaRL_2
!O     !--------------------------------------------------
!O     real(dp) :: L2, L3, L4, LL(0:4)
!O     logical,  parameter :: ready_made = .false.
!O     logical, save :: first = .true.
!O     integer, save      :: nwarn = 0
!O     integer, parameter :: nwarn_limit = 10
!O 
!O     if (L < zero .and. nwarn < nwarn_limit) then
!O        write(0,*) 'WARNING in ma_GetSigmas: L is less than zero: is that safe?'
!O        nwarn = nwarn + 1
!O     end if
!O 
!O     L2 = L * L
!O     L3 = L * L2
!O     L4 = L * L3
!O     LL = (/ one, L, L2, L3, L4 /)
!O 
!O     if (ready_made) then
!O        sigmaR2_1 = sum(mh%F2_1 * LL(0:2))
!O        sigmaR2_2 = sum(mh%F2_2 * LL(0:4))
!O        sigmaRL_1 = sum(mh%FL_1 * LL(0:0))
!O        sigmaRL_2 = sum(mh%FL_2 * LL(0:2))
!O     else
!O     !-- perhaps later on consider storing just the coefficients
!O     !   of the different powers of L...
!O        sigmaR2_1 = (esh%G12 * L2 + esh%G11*L) - mh%P * L + mh%C2_1
!O        sigmaR2_2 = (half*(esh%G12 * L2)**2 + (esh%G23+esh%G11*esh%G12)*L3&
!O             &     +(half*esh%G11**2+esh%G22)*L2) &
!O             &     +(-esh%G12 * mh%P * L3 + (half*mh%PxP - esh%G11*mh%P -pi&
!O             &         *beta0 *mh%P)*L2)& 
!O             &     +mh%C2_1 * (esh%G12*L2 + esh%G11*L)
!O        
!O        sigmaRL_1 = mh%CL_1
!O        sigmaRL_2 = mh%CL_1 * (esh%G12*L2 + esh%G11*L)
!O     end if
!O     
!O     if (mh%norm /= one) then
!O        sigmaR2_1 = sigmaR2_1 * mh%norm
!O        sigmaR2_2 = sigmaR2_2 * mh%norm
!O        sigmaRL_1 = sigmaRL_1 * mh%norm
!O        sigmaRL_2 = sigmaRL_2 * mh%norm
!O     end if
!O     
!O   end subroutine ma_GetSigmas
!O   
!O end module matching_old
