!======================================================================
!
! Things for dealing with the disparate aspects of the resummation
! Intended to guarantee a simple (?!) interface for the poor user.
!
! NB: EvlnType is simply a more logical name qtabid; resummation_definitions 
!     sets up the equivalence for the parameters
!
!
! $Id: resummation_manager.f90,v 1.18 2002/05/23 14:40:07 salam Exp $
!======================================================================
module resummation_manager
  use types; use consts_dp
  use fo_evshp; use matching; use pdf_tabulate; use alfas_server
  use assertions; use warnings_and_errors; use resummed_results
  use resummation_definitions; use brdzE
  use qcd
  implicit none
  private
  
  public :: rm_FOEvshpResum

  !-- have these global in case two separate routine ever used...
  type(qtab_cache)   :: qtcache
  logical            :: qtcache_inited = .false.
  integer, parameter :: cache_size = 20

  !--- scale down to which to evolve ---
  real(dp), parameter :: Qlo = 0.8_dp
  !real(dp), parameter :: Qlo = 0.5_dp
  !--- for power corrections
  real(dp), parameter :: milan_factor = 1.490_dp

contains
  !-------------------------------------------------------------------
  ! Bear in mind possibility of adding MANY new options!
  !
  ! alfas, if given is to be understood to have been evaluated at
  ! scale evshp%Q * muR_Q.
  subroutine rm_FOEvshpResum(evshp, var_vals, sigma_vals, ash, alfas,&
       & muR_Q, MatchType, EvlnType, PowerType, alfa0, muI)
    use event_shapes
    type(fixed_order_evshp), intent(in)  :: evshp
    real(dp),                intent(in)  :: var_vals(:)
    real(dp),                intent(out) :: sigma_vals(:)
    type(as_handle), optional, intent(in)  :: ash
    real(dp),        optional, intent(in)  :: alfas
    integer,  optional,      intent(in)  :: EvlnType
    real(dp), optional,      intent(in)  :: muR_Q
    integer,  optional,      intent(in)  :: MatchType
    integer,  optional,      intent(in)  :: PowerType
    real(dp), optional,      intent(in)  :: alfa0, muI
    !---- defaults -----------------------------------------
    integer,  parameter :: qtabid_def = qtabid_evlnNLO
    integer,  parameter :: MatchType_def = MatchType_basicR
    integer,  parameter :: PowerType_def = PowerType_shift
    real(dp), parameter :: muI_def = 2.0_dp
    integer  :: MatchType_lcl, PowerType_lcl
    real(dp) :: muI_lcl
    !---- qtab stuff -----------------------------------------
    integer, save :: warn_id = warn_id_INIT
    type(qtab_holder)  :: qtabh
    integer            :: qtabid_lcl
    logical            :: need_qtab
    !--- other stuff -----
    real(dp) :: muR_Q_lcl, alfas_lcl, disti1, disti2
    integer  :: i, n
    real(dp) :: var_vals_lcl(size(var_vals))
    real(dp) :: shift(size(var_vals)), shifti, maxvar_NP, sigma_norm
    type(RMaxInfo) :: RMI


    n = assert_eq(size(var_vals), size(sigma_vals), &
         &'rm_FOEvshpResum: var_vals and sigma_vals must be same size')

    if (.not. qtcache_inited) call qtab_InitCache(qtcache,Qlo,cache_size)
    qtcache_inited = .true. 

    muR_Q_lcl = default_or_opt(one, muR_Q)
    qtabid_lcl = default_or_opt(qtabid_def, EvlnType)
    MatchType_lcl = default_or_opt(MatchType_def, MatchType)
    PowerType_lcl = default_or_opt(PowerType_def, PowerType)

    !--- establish what to do with alfas ----------------
    if (present(alfas) .eqv. present(ash)) call wae_error(&
         &'rm_FOEvshpResum: user must supply EITHER alfas OR ash')
    if (present(alfas)) then
       alfas_lcl = alfas
    else
       alfas_lcl = as_value(ash, evshp%Q * muR_Q_lcl)
    end if
    

    !-- For variables with structure function evolution -----------------
    ! With f77 interface there is a problem accessing mh it seems
    ! must investigate to see what is going on...
    ! Maybe forgot a vital initialisation call...
    !write(0,*) evshp%mh%Q
    need_qtab = (evshp%mh%es_info%GP /= zero .and. &
         &          MatchType_lcl /= MatchType_NoResum)
    if (need_qtab) then
       !-- CACHED version; with LO evolution it is much faster
       !   if there are many calls.
       call qtab_GetQTH(qtcache, qtabh, evshp%xbj, evshp%Q*evshp%muF_Q, &
            & alfas_lcl, evshp%Q * muR_Q_lcl, muR_Q_lcl, &
            & qtabid_lcl, evshp%pdf_id, evshp%sh, evshp%pdf,&
            & evshp%esh, evshp%ybj)
    end if

    !-- Deal with the power correction ---------------------------------
    if (PowerType_lcl /= PowerType_none) then
       if (.not.present(alfa0)) call wae_error('rm_FOEvshpResum:&
            &alfa0 not supplied, but needed')
       muI_lcl = default_or_opt(muI_def, muI)
       shift =  evshp%mh%es_info%c_V * &
           & 4.0_dp * cf / pi**2 * muI_lcl/evshp%Q * milan_factor * &
           & ( alfa0 - alfas_lcl - two*beta0*alfas_lcl**2*(&
           &log(muR_Q_lcl*evshp%Q/muI_lcl) + cmw_k/(two*twopi_beta0) + one))  

       !-- special treatment is needed for the broadening(s)
       if (evshp%evshp_id == EvshpId_brd_zE) then
          do i = 1, size(var_vals)
             !-- original form
             shift(i) = shift(i) * brdzE_Power(var_vals(i), &
                  &alfas_lcl,evshp%mh%es_info%esmax)
             !-- do one iteration... (Effect is minimal -> +0.001)
!!$             shifti = shift(i) * brdzE_Power(var_vals(i), &
!!$                  &alfas_lcl,evshp%mh%es_info%esmax)
!!$             shifti = shift(i) * brdzE_Power(var_vals(i)-shifti, &
!!$                  &alfas_lcl,evshp%mh%es_info%esmax)
!!$             shift(i) = shift(i) * brdzE_Power(var_vals(i)-shifti, &
!!$                  &alfas_lcl,evshp%mh%es_info%esmax)
          end do
       end if
       
       maxvar_NP = evshp%mh%es_info%esmax

       select case (PowerType_lcl)
       case(PowerType_shift)
          var_vals_lcl = var_vals - shift
       case(PowerType_modshift)
          var_vals_lcl = var_vals - shift*(one &
               &- (var_vals/maxvar_NP)**evshp%mh%es_info%esNPModPower)
       case default
          call wae_error('rm_FOEvshpResum: illegal PowerType')
       end select
       !write(0,*) 'shift is ', shift
    else
       var_vals_lcl = var_vals
    end if
    

    !--- now actually set about calculating things ------------------------
    ! will only need RMI if any matching gets done; do not set it up
    ! it matching is not needed because the absence of qtabh in such a 
    ! situation could lead to disasters.
    if (MatchType_lcl /= MatchType_NoResum) then
       RMI = rr_RMaxInfo(evshp%mh, qtabh, alfas_lcl, muR_Q_lcl, &
            & disti1_max = evshp%mh%F2L_1 - evshp%sigElim(1),&
            & disti2_max = -evshp%sigElim(2))
    end if
    
    do i = 1, n
       if (var_vals_lcl(i) <= zero) then
          sigma_vals(i) = zero
          cycle
       end if
       call foe_DistIntegrated(evshp, var_vals_lcl(i), disti1, disti2)
       sigma_vals(i) =  rr_R(evshp%mh, qtabh, var_vals_lcl(i), alfas_lcl,&
               & disti1, disti2, muR_Q_lcl, MatchType_lcl, RMI)
    end do

    !-- unnecessary warning?
    !call wae_warn(1, warn_id, 'rm_FOEvshpResum:&
    !     & renorm. to NLO X-sctn minus events with E < Elim')

!!$    !-- alternative, not default; effect on fit is at 1/1000 level
!!$    !   on alpha_s (but perhaps more exactly NLO?)
!!$    sigma_norm = (one + alfas_lcl/twopi*(evshp%mh%F2L_1 - evshp%sigElim(1)))
    
!!$    !-- this should be default
    sigma_norm = (one + alfas_lcl/twopi*(evshp%mh%F2L_1 - evshp%sigElim(1)) &
      & + (alfas_lcl/twopi)**2*(-evshp%sigElim(2) &
      &              + two*twopi_beta0*log(muR_Q_lcl)&
      &                   *(evshp%mh%F2L_1 - evshp%sigElim(1))))


    sigma_vals = sigma_vals / sigma_norm


    !write(0,*) '-----', sigma_norm

    !-- clean up --------------------------
    if (need_qtab) call qtab_DelQTAB(qtabh)
  end subroutine rm_FOEvshpResum
  
end module resummation_manager
